package org.ow2.odis.admin.options;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jdom.Element;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IAction;
import org.ow2.odis.admin.Launch;
import org.ow2.odis.model.Const;

/**
 * The Class AddActivateFileNode.
 * @author Olivier Beyler
 */
public class AddActivateFileNode implements IAction {
    
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(
            AddActivateFileNode.class.getName());

    /** The list of node active. */
    private List lsNodeActive = new ArrayList();

    /** The active file node. */
    private boolean activeFileNode = false;

    /**
     * Actions.
     * @param index the index
     * @param arg the arg
     * @return the int
     * @see com.francetelecom.odis.component.admin.IAction#actions(int, java.lang.String[])
     */
    public int actions(int index, String[] arg) {
        if ("-act".equals(arg[index])) {
            activeFileNode = true;
            // add a new file to activate node
            index++;
            String fileName = arg[index];
            try {
                RandomAccessFile fichier = new RandomAccessFile(fileName, "r");
                String line = fichier.readLine();
                while (line != null) {
                    line = line.trim();
                    if (line.length() != 0 && !line.startsWith("#")) {
                        lsNodeActive.add(line);
                    }
                    line = fichier.readLine();
                }
            } catch (FileNotFoundException e) {
                LOGGER.log(BasicLevel.ERROR, "Can't find file :" + fileName);
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IO Exception on file :"
                        + fileName);
            }
            index++;
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "add a new activate File :"
                        + arg[index]);
            }
        }
        return index;
    }

    /**
     * Gets the descriptions.
     * @return the descriptions
     * @see com.francetelecom.odis.component.admin.IAction#getDescriptions()
     */
    public String getDescriptions() {
        return "-act fileName : adds an activation file";
    }

    /**
     * Post declare.
     * @see com.francetelecom.odis.component.admin.IAction#postDeclare()
     */
    public void postDeclare() {
        if (!activeFileNode) {
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG,
                        "none activate file are declared...");
                LOGGER.log(BasicLevel.DEBUG, "by default all node are active");
            }
            return;
        }
        Element root = Launch.doc.getRootElement();
        // recupere les elements du document maitre
        Element eltNodes = root.getChild(Const.NODES);
        List lsEltNodeToRemove = new ArrayList();
        Element node;
        if (eltNodes != null) {
            List lsEltNode = eltNodes.getChildren(Const.NODE);
            Iterator itNode = lsEltNode.iterator();
            while (itNode.hasNext()) {
                node = (Element) itNode.next();
                String name = node.getAttributeValue(Const.NAME);
                if (!lsNodeActive.contains(name)) {
                    lsEltNodeToRemove.add(node);
                }
            }
        }
        while (!lsEltNodeToRemove.isEmpty()) {
            node = (Element) lsEltNodeToRemove.remove(0);
            eltNodes.removeContent(node);
        }
    }
}
