package org.ow2.odis.admin.options;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IAction;
import org.ow2.odis.admin.PluginsRegistry;

/**
 * The Class AddPlugins.
 * @author olivier beyler - orange-labs
 */
public class AddPlugins implements IAction {
    
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(
            AddPlugins.class.getName());

    /**
     * Actions.
     * @param index the index
     * @param arg the arg
     * @return the int
     * @see com.francetelecom.odis.component.admin.IAction#actions(int, java.lang.String[])
     */
    public int actions(int index, String[] arg) {
        String pluginName = "";
        if ("-plugin".equals(arg[index])) {
            index++;
            try {
                pluginName = arg[index];
                PluginsRegistry.getInstance().addPlugin(pluginName);
            } catch (ClassCastException e) {
                LOGGER.log(BasicLevel.ERROR, "plugin class " + pluginName
                        + " is not a plugin");
            } catch (Exception e) {
                LOGGER.log(BasicLevel.ERROR, "plugin class " + pluginName
                        + " error ", e);
            }
            index++;
        }
        return index;
    }

    /**
     * Gets the descriptions.
     * @return the descriptions
     * @see com.francetelecom.odis.component.admin.IAction#getDescriptions()
     */
    public String getDescriptions() {
        return "-plugin       : add a new plugin";
    }

    /**
     * Post declare.
     * @see com.francetelecom.odis.component.admin.IAction#postDeclare()
     */
    public void postDeclare() {
    }
}
