package org.ow2.odis.admin.options;

import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IAction;
import org.ow2.odis.admin.Launch;


/**
 * The Class DisplayConfig.
 * @author olivier beyler - orange-labs
 */
public class DisplayConfig implements IAction {
    
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize()
            .getLogger(CONSOLE);

    /**
     * If true, it will displays the final configuration.<br>
     * Denotes that the <code>-display</code> option has been used.<br>
     * It is built by concatenating all configuration files (<code>-add</code> option) and disabling the nodes which are not referenced in
     * an activation file (<code>-act</code> option), if activation files are used.
     */
    private boolean showConfig = false;

    /**
     * Actions.
     * @param index the index
     * @param arg the arg
     * @return the int
     * @see com.francetelecom.odis.component.admin.IAction#actions(int, java.lang.String[])
     */
    public int actions(int index, String[] arg) {
        if ("-display".equals(arg[index])) {
            showConfig = true;
            index++;
        }
        return index;
    }

    /**
     * Gets the descriptions.
     * @return the descriptions
     * @see com.francetelecom.odis.component.admin.IAction#getDescriptions()
     */
    public String getDescriptions() {
        return "-display      : displays the final launch configuration on console";
    }

    /**
     * Post declare.
     * @see com.francetelecom.odis.component.admin.IAction#postDeclare()
     */
    public void postDeclare() {
        if (showConfig) {
            try {
                if (LOGGER.isLoggable(BasicLevel.INFO)) {
                    LOGGER.log(BasicLevel.INFO, SEPARATOR);
                    LOGGER.log(BasicLevel.INFO,
                            " Final launching configuration");
                    LOGGER.log(BasicLevel.INFO, SEPARATOR);
                }
                // On utilise ici un affichage classique avec getPrettyFormat()
                XMLOutputter sortie = new XMLOutputter(Format.getPrettyFormat());
                sortie.output(Launch.doc, System.out);
                if (LOGGER.isLoggable(BasicLevel.INFO)) {
                    LOGGER.log(BasicLevel.INFO, SEPARATOR);
                }
            } catch (java.io.IOException e) {
                LOGGER.log(BasicLevel.ERROR, "io exception occured", e);
            }
        }
    }
}
