package org.ow2.odis.admin.options;

import java.util.Iterator;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IAction;
import org.ow2.odis.admin.Launch;
import org.ow2.odis.model.Const;


/**
 * The Class DisplayHelp.
 * @author olivier beyler - orange-labs
 */
public class DisplayHelp implements IAction {
    
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize()
            .getLogger(CONSOLE);

    /** The display help. */
    private boolean displayHelp = false;

    /**
     * Gets the descriptions.
     * @return the descriptions
     * @see com.francetelecom.odis.component.admin.IAction#getDescriptions()
     */
    public String getDescriptions() {
        return "-h / -help    : displays this help";
    };

    /**
     * Actions.
     * @param index the index
     * @param arg the arg
     * @return the int
     * @see com.francetelecom.odis.component.admin.IAction#actions(int, java.lang.String[])
     */
    public int actions(int index, String[] arg) {
        if ("-h".equals(arg[index]) || "-help".equals(arg[index])) {
            displayHelp = true;
            index++;
        }
        return index;
    }

    /**
     * Post declare.
     * @see com.francetelecom.odis.component.admin.IAction#postDeclare()
     */
    public void postDeclare() {
        if (displayHelp) {
            StringBuffer buffer = new StringBuffer(SEPARATOR).append(Const.EOL);
            buffer.append("HELP").append(Const.EOL);
            buffer.append(SEPARATOR).append(Const.EOL);
            Iterator it = Launch.lsActions.iterator();
            while (it.hasNext()) {
                IAction action = (IAction) it.next();
                String description = action.getDescriptions();
                if (description != null && !description.trim().equals("")) {
                    buffer.append(description).append(Const.EOL);
                }
            }
            buffer.append(SEPARATOR).append(Const.EOL);
            buffer.append("Note :").append(Const.EOL);;
            buffer.append("When using multiple configuration files, ");
            buffer.append("<DOMAIN/> should be the same in all files.");
            buffer.append(Const.EOL);
            buffer.append(SEPARATOR).append(Const.EOL);
            if (LOGGER.isLoggable(BasicLevel.INFO)){
                LOGGER.log(BasicLevel.INFO, buffer.toString());
            } else {
                System.out.println(buffer.toString());
            }
        }
    }
}
