package org.ow2.odis.admin.options;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Iterator;
import java.util.List;

import org.jdom.Element;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IAction;
import org.ow2.odis.admin.Launch;

/**
 * The Class GenActivateFile.
 * @author olivier beyler - orange-labs
 */
public class GenActivateFile implements IAction {
    
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(
            GenActivateFile.class.getName());

    /** Name of the generated activation file. */
    private String filename = "";

    /**
     * If true, it will generate an activation file.<br>
     * Denotes that the <code>-gen</code> option has been used.
     */
    private boolean generateActivateFile = false;

    /**
     * (non-Javadoc).
     * @param index the index
     * @param arg the arg
     * @return the int
     * @see com.francetelecom.odis.component.admin.IAction#actions(int, java.lang.String[])
     */
    public int actions(int index, String[] arg) {
        if ("-gen".equals(arg[index])) {
            index++;
            filename = arg[index];
            index++;
            generateActivateFile = true;
        }
        return index;
    }

    /**
     * (non-Javadoc).
     * @return the descriptions
     * @see com.francetelecom.odis.component.admin.IAction#getDescriptions()
     */
    public String getDescriptions() {
        return "-gen fileName : generates the default activation file";
    }

    /**
     * Post declare.
     * @see com.francetelecom.odis.component.admin.IAction#postDeclare()
     */
    public void postDeclare() {
        if (generateActivateFile) {
            File file = new File(filename + ".act");
            try {
                file.createNewFile();
                RandomAccessFile fichier = new RandomAccessFile(filename
                        + ".act", "rw");
                Element root = Launch.doc.getRootElement();
                List ls = root.getChildren("node");
                Iterator it = ls.iterator();
                while (it.hasNext()) {
                    Object ob = it.next();
                    if (ob instanceof Element) {
                        Element elt = (Element) ob;
                        fichier.writeBytes(elt.getAttributeValue("name"));
                        fichier
                                .writeBytes(System
                                        .getProperty("line.separator"));
                    }
                }
                fichier.close();
            } catch (FileNotFoundException e) {
                LOGGER.log(BasicLevel.ERROR, "Can't find file :"
                        + file.getAbsolutePath());
                e.printStackTrace();
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IO Exception on file :"
                        + file.getAbsolutePath());
                e.printStackTrace();
            }
        }
    }
}
