package org.ow2.odis.admin.options;

import java.net.URL;
import java.util.Properties;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IAction;
import org.ow2.odis.admin.Launch;

/**
 * The Class SetProxy.
 * @author olivier beyler - orange-labs
 */

public class SetProxy implements IAction {
    
    /** The Constant CST_PROXY. */
    private static final String CST_PROXY = "-proxy";

    /** The Constant CST_PROXY_DEF. */
    private static final String CST_PROXY_DEF = "PROXY";

    /** The Constant CST_PROXY_HOST. */
    private static final String CST_PROXY_HOST = "host";

    /** The Constant CST_PROXY_PORT. */
    private static final String CST_PROXY_PORT = "port";

    /** The Constant CST_PROXY_HTTP. */
    private static final String CST_PROXY_HTTP = "HTTP";

    /** The Constant CST_PROXY_HTTPS. */
    private static final String CST_PROXY_HTTPS = "HTTPS";

    /** The Constant CST_PROXY_SOCKS. */
    private static final String CST_PROXY_SOCKS = "SOCKS";

    /** The use proxy. */
    private boolean useProxy = false;
    
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(
            SetProxy.class.getName());

    
    
    /**
     * Actions.
     * @param index the index
     * @param arg the arg
     * @return the int
     * @see com.francetelecom.odis.component.admin.IAction#actions(int, java.lang.String[])
     */
    public int actions(int index, String[] arg) {
        if (CST_PROXY.equals(arg[index])) {
            index++;
            useProxy = true;
        }
        return index;
    }

    /**
     * getter of the description.
     * @return the description.
     * @see com.francetelecom.odis.component.admin.IAction#getDescriptions()
     */    
    public String getDescriptions() {
        return "-proxy        : enable the proxy informations contain into odis.xml.";
    }

    /**
     * If the request has been done during the init, it will set the proxy information.
     * @see IAction
     */
    public void postDeclare() {
        if (useProxy) {
            LOGGER.log(BasicLevel.WARN, "proxy have been requested");
            Properties prop = System.getProperties();
            try {
                URL url = SetProxy.class.getResource("/" + Launch.ODIS_CFG);
                if (url == null) {
                    url = new URL(Launch.ODIS_CFG);
                }
                Document doc = new SAXBuilder().build(url.openStream());
                Element root = doc.getRootElement();
                Element proxy = root.getChild(CST_PROXY_DEF);
                if (proxy != null) {
                    boolean setting = false;
                    Element http = proxy.getChild(CST_PROXY_HTTP);
                    if (http != null) {
                        setting = true;
                        prop.put("http.proxySet", "true");
                        prop.put("http.proxyHost", http
                                .getAttributeValue(CST_PROXY_HOST));
                        prop.put("http.proxyPort", http
                                .getAttributeValue(CST_PROXY_PORT));
                    } else {
                        LOGGER.log(BasicLevel.WARN, "no http proxy set");
                    }
                    Element https = proxy.getChild(CST_PROXY_HTTPS);
                    if (https != null) {
                        setting = true;
                        prop.put("https.proxySet", "true");
                        prop.put("https.proxyHost", http
                                .getAttributeValue(CST_PROXY_HOST));
                        prop.put("https.proxyPort", http
                                .getAttributeValue(CST_PROXY_PORT));
                    } else {
                        LOGGER.log(BasicLevel.WARN, "no https proxy set");
                    }
                    Element socks = proxy.getChild(CST_PROXY_SOCKS);
                    if (socks != null) {
                        setting = true;
                        prop.put("socksProxyHost", socks
                                .getAttributeValue(CST_PROXY_HOST));
                        prop.put("socksProxyPort", socks
                                .getAttributeValue(CST_PROXY_PORT));
                    } else {
                        LOGGER.log(BasicLevel.WARN, "no SOCK proxy set");
                    }
                    if (!setting) {
                        StringBuffer err = new StringBuffer("-proxy option ");
                        err.append("is request but ");
                        err.append("no proxy information found into \"");
                        err.append(Launch.ODIS_CFG);
                        err.append("\" file.");
                        LOGGER.log(BasicLevel.ERROR, err.toString());
                    }
                }
            } catch (JDOMException e) {
                LOGGER.log(BasicLevel.ERROR, "Init of proxy failed", e);
            } catch (Exception e) {
                // IOException or Null pointer exception (URL = null)
                // have the same behavior
                StringBuffer warn = new StringBuffer("No or bad file \"");
                warn.append(Launch.ODIS_CFG);
                warn.append("\" present into classpath.");
                LOGGER.log(BasicLevel.WARN, warn.toString());
            }
        }
    }
}
