package org.ow2.odis.admin.options;

import java.net.URL;
import java.util.Properties;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.admin.IAction;
import org.ow2.odis.admin.Launch;

/**
 * The Class SetTrustAnchors.
 * @author olivier beyler - orange-labs
 */
public class SetTrustAnchors implements IAction {
    
    /** The Constant CST_TRUSTANCHORS. */
    private static final String CST_TRUSTANCHORS = "-trustAnchors";

    /** The Constant CST_TRUSTANCHORS_DEF. */
    private static final String CST_TRUSTANCHORS_DEF = "TRUSTANCHORS";

    /** The Constant CST_SSL_KEYSTORE. */
    private static final String CST_SSL_KEYSTORE = "KEYSTORE";

    /** The Constant CST_SSL_TRUSTSTORE. */
    private static final String CST_SSL_TRUSTSTORE = "TRUSTSTORE";

    /** The Constant CST_DEBUG. */
    private static final String CST_DEBUG = "DEBUG";

    /** The Constant CST_VALUE. */
    private static final String CST_VALUE = "value";

    /** The Constant CST_PASSWORD. */
    private static final String CST_PASSWORD = "password";

    /** The use trust anchor. */
    private boolean useTrustAnchor = false;

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(
            SetTrustAnchors.class.getName());

    /**
     * Actions.
     * @param index the index
     * @param arg the arg
     * @return the int
     * @see com.francetelecom.odis.component.admin.IAction#actions(int, java.lang.String[])
     */
    public int actions(int index, String[] arg) {
        if (CST_TRUSTANCHORS.equals(arg[index])) {
            index++;
            useTrustAnchor = true;
        }
        return index;
    }

    /**
     * Gets the descriptions.
     * @return the descriptions
     * @see com.francetelecom.odis.component.admin.IAction#getDescriptions()
     */
    public String getDescriptions() {
        return CST_TRUSTANCHORS
                + "        : enable the trust anchor informations contain into odis.xml.";
    }

    /**
     * If the request has been done during the init, it will set the proxy information.
     * @see IAction
     */
    public void postDeclare() {
        if (useTrustAnchor) {
            LOGGER.log(BasicLevel.WARN, "trust anchor have been requested");
            Properties prop = System.getProperties();
            try {
                URL url = SetTrustAnchors.class.getResource("/"
                        + Launch.ODIS_CFG);
                if (url == null) {
                    url = new URL(Launch.ODIS_CFG);
                }
                Document doc = new SAXBuilder().build(url.openStream());
                Element root = doc.getRootElement();
                Element trustAnchor = root.getChild(CST_TRUSTANCHORS_DEF);
                if (trustAnchor != null) {
                    boolean setting = false;
                    Element sslKeystore = trustAnchor
                            .getChild(CST_SSL_KEYSTORE);
                    if (sslKeystore != null) {
                        setting = true;
                        prop.put("javax.net.ssl.keyStore", sslKeystore
                                .getAttributeValue(CST_VALUE));
                        prop.put("javax.net.ssl.keyStorePassword", sslKeystore
                                .getAttributeValue(CST_PASSWORD));
                    } else {
                        LOGGER.log(BasicLevel.WARN, "no ssl keystore set");
                    }
                    Element sslTrustStore = trustAnchor
                            .getChild(CST_SSL_TRUSTSTORE);
                    if (sslTrustStore != null) {
                        prop.put("javax.net.ssl.trustStore", sslTrustStore
                                .getAttributeValue(CST_VALUE));
                        prop.put("javax.net.ssl.trustStorePassword",
                                sslTrustStore.getAttributeValue(CST_PASSWORD));
                    } else {
                        LOGGER.log(BasicLevel.WARN, "no ssl truststore set");
                    }
                    Element debug = trustAnchor.getChild(CST_DEBUG);
                    if (debug != null) {
                        setting = true;
                        prop.put("javax.net.debug", debug
                                .getAttributeValue(CST_VALUE));
                    } else {
                        LOGGER.log(BasicLevel.WARN, "no debug set");
                    }
                    if (!setting) {
                        StringBuffer err = new StringBuffer(CST_TRUSTANCHORS);
                        err
                                .append(" option is request but no trust anchors information found into \"");
                        err.append(Launch.ODIS_CFG);
                        err.append("\" file.");
                        LOGGER.log(BasicLevel.ERROR, err.toString());
                    }
                }
            } catch (JDOMException e) {
                LOGGER.log(BasicLevel.ERROR, "Init of trus anchors failed", e);
            } catch (Exception e) {
                // IOException or Null pointer exception (URL = null)
                // have the same behavior
                StringBuffer warn = new StringBuffer("No or bad file \"");
                warn.append(Launch.ODIS_CFG);
                warn.append("\" present into classpath.");
                LOGGER.log(BasicLevel.WARN, warn.toString());
            }
        }
    }
}
