package org.ow2.odis.bufferIn;

import java.util.Collections;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.context.Context;

/**
 * The Class SynchronizedObject.
 */
public class BufferListObject {
    /**
     * The Class SynchronizedObjectException.
     */
    class SynchronizedObjectException extends RuntimeException {
        /** The Constant serialVersionUID. */
        private static final long serialVersionUID = 1L;

        /**
         * Constructor.
         * @param chaine the chaine
         */
        public SynchronizedObjectException(String chaine) {
            super(chaine);
        }
    }

    /** The last object. */
    transient Object lastObject = null;

    /** The size. */
    final transient int size;

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            BufferListObject.class.getName());

    /** la liste des objets a traiter. */
    transient List objects = null;

    /** identifiant de l objet dont est issu le message. */
    transient String idObject = "";

    /** The context. */
    private transient Context context;

    /**
     * Constructor.
     * @param pId identity
     * @param pMsg byte array
     * @param pObjects the objects
     * @param context the context
     */
    public BufferListObject(String pId, byte[] pMsg, List pObjects,
            Context context) {
        super();
        idObject = pId;
        objects = Collections.synchronizedList(pObjects);
        size = pMsg.length;
        this.context = context;
    }

    /**
     * Gets the object.
     * @return the first object contain into the list
     */
    public Object getObject() {
        Object result = null;
        if (objects == null) {
            if (LOGGER.isLoggable(BasicLevel.WARN)) {
                LOGGER.log(BasicLevel.WARN,
                        "try to access a deleted SynchronizedObject");
            }
        } else if (objects.isEmpty()) {// plus d objet a traiter
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG,
                        "no more object into SynchronizedObject");
            }
        } else {
            lastObject = objects.get(0);
            result = lastObject;
        }
        return result;
    }

    /**
     * Release object.
     * @param object the object
     */
    public void releaseObject(final Object object) {
        if (lastObject == object) {
            final boolean remove = objects.remove(object);
            if (!remove) {
                LOGGER.log(BasicLevel.ERROR, "impossible a release");
                throw new SynchronizedObjectException(
                        "try to release un unknown object");
            }
            update();
        } else {
            LOGGER.log(BasicLevel.FATAL, "try to release an unknown object");
        }
    }

    /**
     * give the size of the object list.
     * @return the remainding objects
     */
    int getRemaindingObjects() {
        return objects.size();
    }

    /**
     * Update.
     * @return true
     */
    public boolean update() {
        return true;
    }

    /**
     * destroy synchronizedObject.
     * @return true si le support a ete reellement efface false dans les autres cas
     */
    public boolean delete() {
        idObject = null;
        objects = null;
        return true;
    }

    /**
     * Gets the context.
     * @return the context
     */
    public Context getContext() {
        return context;
    }
}
