package org.ow2.odis.bufferIn;

import java.io.File;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.odis.connection.context.Context;

/**
 * The Class PersistenceSynchronizedObjectFactory.
 * @author olivier beyler - orange-labs
 */
public class PersistenceBufferListObjectFactory extends BufferListObjectFactory {
    /** path directory of persistence. */
    private String path = "";

    /** The identity. */
    long identity = System.currentTimeMillis();

    /** The set. */
    Set setPathAlreadyCreated = new HashSet();

    /**
     * Instantiates a new persistence synchronized object factory.
     * @param path the path
     */
    public PersistenceBufferListObjectFactory(String path) {
        super();
        final StringBuffer buffer = new StringBuffer(path);
        buffer.append(File.separator);
        buffer.append("IN");
        this.path = buffer.toString();
        // Creation du path au besoin
        File file = new File(path);
        file.mkdirs();
    }

    /**
     * effectue la creation d un object persistant.
     * @param id the id
     * @param msg the msg
     * @param objects the objects
     * @param context the context
     * @return the synchronized Object created with the param
     */
    public BufferListObject getObject(String id, byte[] msg, List objects,
            Context context) {
        BufferListObject po = null;
        identity++;
        checkPath(id);
        String name = path + File.separator + id + File.separator + identity
                + ".sav";
        po = new PersistenceBufferListObject(name, id, msg, objects, context,
                true);
        return po;
    }

    /**
     * check if path as already been created.
     * @param id the id
     */
    public void checkPath(String id) {
        if (!setPathAlreadyCreated.contains(id)) {
            File file = new File(path + File.separator + id);
            file.mkdirs();
            setPathAlreadyCreated.add(id);
        }
    }
}
