package org.ow2.odis.bufferOut;

import java.util.List;

import org.ow2.odis.connection.context.Context;

/**
 * This interface is to be implemented by classes that need to be notified when a critical operation on a BufferOut is about to happen.
 * @author ojoh6423
 */
public interface IBufferOutWatcher {
    
    /**
     * This method is called when a context-specific buffer is about to be cleared according to the clearing options defined. The clear
     * operation will actually happen if and only if this method returns true.
     * @param context the context associated with the buffer about to be cleared
     * @param contents the contents of that buffer
     * @param reason the policy value denoting on what basis the buffer is about to be cleared (may be checked against String constants in
     * BufferOutAttribut)
     * @return true, if checks if is context buffer out removal allowed
     */
    boolean isContextBufferOutRemovalAllowed(Context context, List contents,
            String reason);
}
