package org.ow2.odis.connection;

import org.ow2.odis.connection.context.IContextualConnectionOut;
import org.ow2.odis.model.AbstractAttribute;
import org.ow2.odis.model.AbstractConnectionAttribute;

/**
 * IConnectionFactory define the interface require for a factory of connection
 * @author olivier beyler - orange-labs
 */
public interface IConnectionFactory {
    /**
     * Creates a new IN connection or delegates the creation of this new connection to a dedicated factory.
     * @param attribute the attribute holding the connection parameters
     * @return a new connection In just created
     */
    IConnectionIn newConnectionIn(final AbstractConnectionAttribute attribute);

    /**
     * Creates a new OUT connection or delegates the creation of this new connection to a dedicated factory.
     * @param attribute the attribute holding the connection parameters
     * @return the connection Out just created
     */
    IConnectionOut newConnectionOut(final AbstractConnectionAttribute attribute);

    /**
     * Creates a new OUT connection or delegates the creation of this new connection to a dedicated factory.
     * @param attribute the attribute holding the connection parameters
     * @return the connection just created
     */
    IContextualConnectionOut newContextualConnectionOut(
            final AbstractConnectionAttribute attribute);

    /**
     * Creates a new connection attribute.
     * @param attribute the attribute of the parent
     * @return a connection attribute
     */
    AbstractConnectionAttribute newAttribute(AbstractAttribute attributes);

}
