package org.ow2.odis.connection.composite;

import org.ow2.odis.connection.AbstractConnectionfactory;
import org.ow2.odis.connection.IConnectionOut;
import org.ow2.odis.connection.context.IContextualConnectionOut;
import org.ow2.odis.model.AbstractAttribute;
import org.ow2.odis.model.AbstractConnectionAttribute;
import org.ow2.odis.model.CompositeConnectionAttribute;

/**
 * implement Factory and singleton design pattern.
 * time.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class CompositeFactory extends AbstractConnectionfactory {
    /**
     * singleton pattern.
     */
    private static CompositeFactory instance = new CompositeFactory();

    /**
     * constructor.
     */
    private CompositeFactory() {
        super();
    }

    /**
     * getter. (implement Singleton pattern)
     * @return unique instance
     */
    public static CompositeFactory getInstance() {
        return instance;
    }

    /**
     * Provides a new outgoing composite, as described by the attribute.
     * @param attribute the CompositeCnxAttribut describing the connection
     * @return the newly made composite
     */
    public IConnectionOut newConnectionOut(final AbstractConnectionAttribute attribute) {
        return new CompositeOut((CompositeConnectionAttribute) attribute);
    }

    /**
     * Composite implements no contextual connection
     * @param attribute an ignored attribute
     * @return always null
     */
    public IContextualConnectionOut newContextualConnectionOut(
            AbstractConnectionAttribute attribute) {
        return new CompositeOut((CompositeConnectionAttribute) attribute, true);
    }

    public AbstractConnectionAttribute newAttribute(final AbstractAttribute attribut) {
        return new CompositeConnectionAttribute(attribut);
    }

    /**
     * getter of type name
     * @return String
     */
    protected String getTypeName() {
        return ("Composite");
    }
}
