package org.ow2.odis.connection.context;
/**
 * This interface is to be implemented by classes that need to be notified by the ContextManager of changes in the set of available
 * contexts.
 * @author ojoh6423
 */
public interface IContextWatcher {
    /**
     * The ContextManager calls this method when a context is no longer associated with a valid session.<br>
     * This method is NOT called when a context (typically implicit) is replaced by a new one (typically explicit). See contextReplaced().<br>
     * Implementing classes should take any appropriate measures. Processing should be light as the call is synchronous.
     * @param ctx the orphaned context
     */
    void contextLost(Context ctx);

    /**
     * The ContextManager calls this method when a context is now associated with a valid session.<br>
     * This method is NOT called when a context (typically implicit) is replaced by a new one (typically explicit). See contextReplaced().<br>
     * Implementing classes should take any appropriate measures. Processing should be light as the call is synchronous.
     * @param ctx the now active context
     */
    void contextSet(Context ctx);

    /**
     * The ContextManager calls this method when a context is replaced with another.<br>
     * This typically happens when business or protocol methods replace an implicit identification (context) by an explicit one after an
     * identification exchange.<br>
     * This method is NOT called when the ContextManager has been asked to override a context-session association with a new one : if a new
     * context overrides another, contextLost() is called for the orphan and contextSet() for the new one ; if a session overrides another,
     * no context notification is made.<br>
     * Implementing classes should take any appropriate measures. Processing should be light as the call is synchronous.
     * @param oldContext the obsolete context value
     * @param newContext the new value
     */
    void contextReplaced(Context oldContext, Context newContext);
}
