package org.ow2.odis.connection.context;

import org.ow2.odis.connection.IConnectionOut;

/**
 * This interface specializes IConnectionOut by providing the ability to send a ContextMessage, i.e. a message with an associated context.<br>
 * The way this context is used is implementation-specific.<br>
 * @author ojoh6423
 */
public interface IContextualConnectionOut extends IConnectionOut {
    /**
     * Sends the payload of the message to the peer associated with the context.
     * @param pMsg the context message, holding payload and context
     * @return true if successful, else false
     */
    boolean send(ContextMessage pMsg);

    /**
     * Registers the fact that the connection will be used for two-way communication.<br>
     * The implementation should take any measures appropriate for that use case.
     */
    void usedAsTwoWay();
}
