package org.ow2.odis.connection.socket;

import java.util.Observable;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.IConnectionOut;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisConnectionException;
import org.ow2.odis.lifeCycle.connection.AbstractConnectionState;
import org.ow2.odis.lifeCycle.connection.StateConnectionLoaded;
import org.ow2.odis.model.AbstractConnectionAttribute;
import org.ow2.odis.model.SocketConnectionAttribute;
import org.ow2.odis.port.PortOut;

/**
 * The Class AbstractTCPConnectionOut.
 */
public abstract class AbstractTCPConnectionOut implements IConnectionOut {
    /** The connection socket. */
    protected AbstractTCPConnectionOut connectionSocket = null;

    /** The port out. */
    protected PortOut portOut = null;

    /** The cnx state. */
    private AbstractConnectionState cnxState = new StateConnectionLoaded();

    /** The attribut. */
    public final SocketConnectionAttribute attribut;

    /**
     * Instantiates a new abstract TCP connection out.
     * @param attribute the attribute
     */
    AbstractTCPConnectionOut(SocketConnectionAttribute attribute) {
        super();
        this.attribut = attribute;
        attribute.addObserver(this);
        connectionSocket = this;
    }

    /** The Constant loggerCnxOut. */
    static final Logger loggerCnxOut = Monolog.initialize().getLogger(
            AbstractTCPConnectionOut.class.getName());

    /**
     * Gets the protocol out.
     * @return the protocol out
     */
    IProtocolOut getProtocolOut() {
        IProtocolOut result = null;
        try {
            result = attribut.createProtocolOutInstance();
        } catch (OdisAttributeException e) {
            loggerCnxOut
                    .log(
                            BasicLevel.FATAL,
                            "protocol out should have been ejected during resolve component!",
                            e);
        }
        return result;
    }

    /**
     * @throws OdisConnectionException
     */
    public void initComponent() throws OdisConnectionException {
        try {
            attribut.checkProperties();
        } catch (OdisAttributeException e) {
            throw new OdisConnectionException(e);
        }
    }

    /**
     */
    public void unloadComponent() {
        attribut.deleteObserver(this);
    }

    /**
     * @param state
     */
    public void setConnectionState(AbstractConnectionState state) {
        this.cnxState = state;
        if (loggerCnxOut.isLoggable(BasicLevel.INFO)) {
            StringBuffer buffer = new StringBuffer(toString());
            buffer.append(" set to ");
            buffer.append(state.getStateName());
            loggerCnxOut.log(BasicLevel.INFO, buffer.toString());
        }
    }

    /**
     * Gets the cnx state.
     * @return AbstractCnxState
     */
    public AbstractConnectionState getConnectionState() {
        return cnxState;
    }

    /**
     * Update.
     * @param o the o
     * @param arg the arg
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg) {
    }

    /**
     * Gets the attribut.
     * @return AbstractCnxAttribut
     */
    public AbstractConnectionAttribute getAttribute() {
        return attribut;
    }
}
