package org.ow2.odis.connection.socket;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.model.SocketConnectionAttribute;
import org.ow2.odis.util.CodingDecoding;
import org.ow2.odis.util.IntegerModif;

/**
 * This protocole is design to transform a stream to message. the separator used by this protocol is the end of a line.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class LineProtocol implements IProtocolIn, IProtocolOut {
    /**
     * constant.
     */
    private static final String CST_UTF_8 = "UTF-8";

    /**
     * define the encoding type used by this protocol.
     */
    private String encoding = CST_UTF_8;

    /**
     * Keep a same stream reader on each call of read
     */
    private InputStreamReader streamReader = null;

    /**
     * keep the same bufferReader on each call of read
     */
    private BufferedReader bufferReader = null;

    /**
     * logger of this class.
     */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            LineProtocol.class.getName());

    /**
     * nothing can
     * @return TRUE
     */
    public boolean isStreamStillOpen() {
        return true;
    }

    /**
     * Constructor.
     */
    public LineProtocol() {
    }

    /**
     * Reader.
     * @param input InputStream
     * @param output OutputStream
     * @return corresponding byte[] message
     * @throws IOException in case of broken stream
     */
    public byte[] read(final InputStream input, final OutputStream output)
            throws IOException {
        if (streamReader == null) {
            streamReader = new InputStreamReader(input, encoding);
            bufferReader = new BufferedReader(streamReader, 1);
        }
        String buffer = bufferReader.readLine();
        if (buffer == null) {
            buffer = "";
        }
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "read message: " + buffer);
        }
        return CodingDecoding.stringToBytes(buffer);
    }

    /**
     * Writer.
     * @param input InputStream
     * @param output OutputStream
     * @param message the message
     * @return true if succeed
     * @throws IOException in case of broken stream
     */
    public boolean write(InputStream input, OutputStream output, byte[] message)
            throws IOException {
        final IntegerModif index = new IntegerModif(0);
        String text;
        while (index.intValue() < message.length) {
            text = CodingDecoding.getString(message, index);
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "writing message:" + message);
            }
            output.write(text.getBytes());
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "writing \\n");
            }
            output.write('\n');
        }
        // output.write(EOT);
        return true;
    }

    /**
     * nothing is done
     * @param attribute SocketCnxAttribut
     */
    public void setSocketCnxAttribut(SocketConnectionAttribute attribute) {
        // nothing to do
    }
}
