package org.ow2.odis.connection.socket;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.context.Context;

/**
 * look on the socket if a new message is arrived.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class MessageListener extends Thread {
    /** keep a socket reference. */
    Socket socket = null;

    /** keep a protocol reference. */
    private IProtocolIn inProtocol = null;

    /** logger of this class/. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            MessageListener.class.getName());

    /** if an error occurred stopListening is fixed to true. */
    public boolean stopListening = false;

    /** keep a server reference. */
    private AbstractTCPConnectionIn server = null;

    /**
     * Constructor.
     * @param pServer the server
     */
    MessageListener(AbstractTCPConnectionIn pServer) {
        super("TCPClientListener");
        this.server = pServer;
    }

    /** The context. */
    Context context;

    /**
     * Run.
     * @see java.lang.Thread#run()
     */
    public void run() {
        server.attribut.incNumberConnection(1);
        try {
            try {
                InputStream streamIn = getSocket().getInputStream();
                OutputStream streamOut = getSocket().getOutputStream();
                while (inProtocol.isStreamStillOpen()) {
                    byte[] msg = inProtocol.read(streamIn, streamOut);
                    // <JDO
                    // "This should not happen but this test will help debugging if it does..."
                    // />
                    if (msg.length == 0) {
                        LOGGER.log(BasicLevel.ERROR,
                                "Zero length message read on socket - ignoring //"
                                        + hashCode());
                        LOGGER.log(BasicLevel.ERROR, "Timeout was "
                                + socket.getSoTimeout());
                    } else {
                        server.addMessage(msg, context);
                        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                            LOGGER.log(BasicLevel.DEBUG,
                                    "Normal message read on socket //"
                                            + hashCode());
                        }
                    }
                }
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IOException on read protocol", e);
            } catch (InterruptedException e) {
                LOGGER.log(BasicLevel.INFO,
                        "interrupt exception during read protocol");
            }
            try {
                socket.close();
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IOException during closing connection",
                        e);
            }
        } catch (RuntimeException e) {
            LOGGER.log(BasicLevel.ERROR, "Listen exception :", e);
        }
        LOGGER.log(BasicLevel.ERROR, "END LISTENING //" + hashCode());
        stopListening = true;
        server.attribut.incNumberConnection(-1);
    }

    /**
     * Gets the socket.
     * @return Socket.
     */
    public Socket getSocket() {
        return socket;
    }

    /**
     * Sets the socket.
     * @param pSocket Socket
     */
    public void setSocket(Socket pSocket) {
        this.socket = pSocket;
        this.context = new Context(socket.getInetAddress().toString(), ""
                + socket.getPort());
    }

    /**
     * Sets the protocol.
     * @param protocol IprotocolIn
     */
    public void setProtocol(IProtocolIn protocol) {
        this.inProtocol = protocol;
    }
}
