package org.ow2.odis.connection.socket;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.ow2.odis.model.SocketConnectionAttribute;
import org.ow2.odis.util.CodingDecoding;
import org.ow2.odis.util.IntegerModif;
import org.ow2.odis.util.StreamReader;

/**
 * default protocol use by socket when no protocole is set. This protocol only send new message when the last has been acknoledge.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class OdisProtocol implements IProtocolIn, IProtocolOut {
    /** constant. */
    private static final byte ANSWERS = '@';

    /** Stream reader. */
    private StreamReader reader = null;

    /**
     * Instantiates a new odis protocol.
     */
    public OdisProtocol() {
        reader = new StreamReader();
    }

    /**
     * Read.
     * @param input the input
     * @param output the output
     * @return the byte[]
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public byte[] read(InputStream input, OutputStream output)
            throws IOException {
        int size;
        byte[] buffer;
        byte[] sizeBuffer = reader.read(input, 4);
        IntegerModif index = new IntegerModif(0);
        size = CodingDecoding.getIntFrom4bytes(sizeBuffer, index);
        buffer = reader.read(input, size);
        output.write(ANSWERS);
        return buffer;
    }

    /**
     * Checks if is stream still open.
     * @return true, if is stream still open
     */
    public boolean isStreamStillOpen() {
        return true;
    }

    /**
     * Sets the socket connection attribute.
     * @param attribute the attribute
     */
    public void setSocketCnxAttribut(SocketConnectionAttribute attribute) {
    }

    /**
     * Write.
     * @param input the input
     * @param output the output
     * @param message the message
     * @return true, if write
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public boolean write(InputStream input, OutputStream output, byte[] message)
            throws IOException {
        byte[] size = CodingDecoding.intTo4Bytes(message.length);
        byte[][] encode = new byte[2][];
        encode[0] = size;
        encode[1] = message;
        output.write(CodingDecoding.getLinearByte(encode));
        byte[] responce = reader.read(input, 1);
        return responce[0] == ANSWERS ? true : false;
    }
}
