package org.ow2.odis.connection.socket;

import org.ow2.odis.connection.AbstractConnectionfactory;
import org.ow2.odis.connection.IConnectionIn;
import org.ow2.odis.connection.IConnectionOut;
import org.ow2.odis.model.AbstractAttribute;
import org.ow2.odis.model.AbstractConnectionAttribute;
import org.ow2.odis.model.Const;
import org.ow2.odis.model.SocketConnectionAttribute;

/**
 * @author olivier beyler - orange-labs
 */
public class SocketConnectionFactory extends AbstractConnectionfactory {
    static SocketConnectionFactory instance = new SocketConnectionFactory();

    private SocketConnectionFactory() {
        super();
    }

    /**
     * getter. (implement Singleton pattern)
     * @return unique instance
     */
    public static SocketConnectionFactory getInstance() {
        return instance;
    }

    /**
     * @param attribut
     * @return
     */
    public IConnectionIn newConnectionIn(AbstractConnectionAttribute attribut) {
        IConnectionIn result;
        SocketConnectionAttribute socketAttribut = (SocketConnectionAttribute) attribut;
        if (Const.PROPERTY_IN.equals(socketAttribut.getServerSide())) {
            result = new TCPServerConnectionIn((SocketConnectionAttribute) attribut);
        } else {
            result = new TCPClientConnectionIn((SocketConnectionAttribute) attribut);
        }
        return result;
    }

    public IConnectionOut newConnectionOut(AbstractConnectionAttribute attribut) {
        IConnectionOut result;
        SocketConnectionAttribute socketAttribut = (SocketConnectionAttribute) attribut;
        if (Const.PROPERTY_OUT.equals(socketAttribut.getServerSide())) {
            result = new TCPServerConnectionOut((SocketConnectionAttribute) attribut);
        } else {
            result = new TCPClientConnectionOut((SocketConnectionAttribute) attribut);
        }
        return result;
    }

    public AbstractConnectionAttribute newAttribute(AbstractAttribute attribut) {
        return new SocketConnectionAttribute(attribut);
    }

    /**
     * @return
     */
    protected String getTypeName() {
        return ("Socket");
    }
}
