package org.ow2.odis.connection.socket;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisConnectionException;
import org.ow2.odis.model.SocketConnectionAttribute;

/**
 * This module provide the ability to accept TCP connection and instantiate the client witch manage this connection.
 * @author O.Beyler
 */
public class TCPServerConnectionIn extends AbstractTCPConnectionIn {
    /** logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            TCPServerConnectionIn.class.getName());

    /** The srv. */
    private ServerSocket srv;

    /** The tcp server. */
    private TCPServerSocket tcpServer;

    /** The running. */
    boolean running = true;

    /**
     * Instantiates a new TCP server connection in.
     * @param attribute the attribute
     */
    public TCPServerConnectionIn(SocketConnectionAttribute attribute) {
        super(attribute);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "Create TCPServerConnection");
        }
    }

    /**
     * inner class.
     * @author Beyler Olivier FranceTelecom R&D
     */
    class TCPServerSocket extends Thread {
        /**
         * Constructor.
         */
        public TCPServerSocket() {
            super();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "Create TCPServer");
            }
        }

        /**
         * open a socket server.
         */
        public void init() {
            try {
                srv = new ServerSocket(attribut.getPort());
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, "create ServerSocket on port "
                            + attribut.getPort());
                }
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR,
                        "Unable to create ServerSocket on port "
                                + attribut.getPort(), e);
            }
        };

        /**
         * loop until running is true.
         * @see java.lang.Thread#run()
         */
        public void run() {
            try {
                Socket connection;
                MessageListener listener;
                while (running) {
                    if (LOGGER.isLoggable(BasicLevel.INFO)) {
                        LOGGER
                                .log(BasicLevel.INFO,
                                        "server wait new connection");
                    }
                    connection = srv.accept();
                    connection.setSoTimeout(attribut.getTimeOut());
                    if (LOGGER.isLoggable(BasicLevel.INFO)) {
                        LOGGER.log(BasicLevel.INFO,
                                "server accept new connection");
                    }
                    listener = new MessageListener(connectionSocket);
                    listener.setSocket(connection);
                    try {
                        listener.setProtocol(attribut
                                .createProtocolInInstance());
                    } catch (OdisAttributeException e) {
                        throw new RuntimeException(e);
                    }
                    listener.start();
                }
            } catch (RuntimeException e) {
                LOGGER.log(BasicLevel.ERROR, "RuntimeException", e);
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IOException", e);
            }
            try {
                srv.close();
            } catch (IOException e) {
                LOGGER.log(BasicLevel.WARN, "Exception on socketServer close",
                        e);
            }
        }
    }

    /**
     * @throws OdisConnectionException
     */
    public void launchComponent() throws OdisConnectionException {
        tcpServer = new TCPServerSocket();
        tcpServer.init();
        tcpServer.start();
    }

    /**
     */
    public void stopComponent() {
        running = false;
        tcpServer.interrupt();
        try {
            if (srv != null) {
                srv.close();
            }
        } catch (IOException e) {
            LOGGER.log(BasicLevel.DEBUG, "Closing socket failed...");
        }
    }
}
