package org.ow2.odis.decoder;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.StreamCorruptedException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.util.CodingDecoding;
import org.ow2.odis.util.IntegerModif;

/**
 * Implementation of a generic decoder witch is able to decode the message created by generic encoder. The only requierment for the object
 * is : they must be serializable.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class GenericDecoder implements IDecoder {
    /**
     * logger of this class.
     */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            GenericDecoder.class.getName());

    /**
     * Decode.
     * @param msg the msg
     * @return the list
     * @see com.francetelecom.odis.component.decoder.IDecoder#decode(byte[])
     */
    public List decode(byte[] msg) {
        List result = new ArrayList();
        IntegerModif index = new IntegerModif(0);
        try {
            while (index.intValue() < msg.length) {
                byte[] temp = CodingDecoding.getBytesSecure(msg, index);
                if (temp.length > 0) {
                    ByteArrayInputStream byteStream = new ByteArrayInputStream(
                            temp);
                    ObjectInputStream objectStream = null;
                    Object object = null;
                    try {
                        objectStream = new ObjectInputStream(byteStream);
                        object = objectStream.readObject();
                        objectStream.close();
                        result.add(object);
                    } catch (java.io.EOFException e) {
                        LOGGER.log(BasicLevel.ERROR, "EOFException", e);
                    } catch (StreamCorruptedException e) {
                        LOGGER.log(BasicLevel.ERROR,
                                "StreamCorruptedException", e);
                    } catch (IOException e) {
                        LOGGER.log(BasicLevel.ERROR, "IOException", e);
                    } catch (ClassNotFoundException e) {
                        LOGGER.log(BasicLevel.ERROR, "ClassNotFoundException",
                                e);
                    } finally {
                        byteStream = null;
                        object = null;
                        objectStream = null;
                    }
                }
            }
        } catch (UnsupportedEncodingException e) {
            LOGGER.log(BasicLevel.ERROR, "UnsupportedEncodingException", e);
        }
        return result;
    }

    /**
     * Gets the type.
     * @return the type
     * @see com.francetelecom.odis.component.decoder.IDecoder#getType()
     */
    public String getType() {
        return Object.class.getName();
    }
}
