package org.ow2.odis.decoder;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.util.CodingDecoding;
import org.ow2.odis.util.IntegerModif;

/**
 * Basic implementation of Text Decoder.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class TextDecoder implements IDecoder {
    /** logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            TextDecoder.class.getName());

    /**
     * Instantiates a new text decoder.
     */
    public TextDecoder() {
        super();
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "create TextDecoder");
        }
    }

    /**
     * Decode.
     * @param msg the msg
     * @return the list
     * @see com.francetelecom.odis.component.decoder.IDecoder#decode(byte[])
     */
    public List decode(byte[] msg) {
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "decoding msg");
        }
        List result = new ArrayList();
        IntegerModif index = new IntegerModif(0);
        try {
            while (index.intValue() < msg.length) {
                result.add(CodingDecoding.getString(msg, index));
            }
        } catch (UnsupportedEncodingException e) {
            LOGGER.log(BasicLevel.ERROR, "encoding exception");
        }
        return (result);
    }

    /**
     * Gets the type.
     * @return the type
     * @see com.francetelecom.odis.component.decoder.IDecoder#getType()
     */
    public String getType() {
        return String.class.getName();
    }
}
