package org.ow2.odis.encoder;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.util.CodingDecoding;

/**
 * This class give the opportunity to use the java serialize without defining a specific encoder or decoder.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class GenericEncoder implements IEncoder {
    /**
     * logger of this class.
     */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            GenericEncoder.class.getName());

    /**
     * encode the object.
     * @param object the object
     * @return a byte array
     * @see com.francetelecom.odis.component.encoder.IEncoder#encode(java.lang.Object)
     */
    public byte[] encode(final Object object) {
        byte[] result = null;
        try {
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            ObjectOutputStream objectsStream = new ObjectOutputStream(
                    byteStream);
            objectsStream.writeObject(object);
            objectsStream.close();
            byte[] temp = byteStream.toByteArray();
            result = CodingDecoding.bytesToBytes(temp);
            temp = null;
            byteStream = null;
            objectsStream = null;
        } catch (IOException e) {
            LOGGER.log(BasicLevel.ERROR, "unable to use GenericEncoder ", e);
        } catch (RuntimeException e) {
            LOGGER.log(BasicLevel.ERROR, "unable to use GenericEncoder ", e);
        }
        return result;
    }

    /**
     * give the type of encoder.
     * @return the type of encoder
     * @see com.francetelecom.odis.component.encoder.IEncoder#getType()
     */
    public String getType() {
        return Object.class.getName();
    }
}
