package org.ow2.odis.encoder;

import java.io.UnsupportedEncodingException;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.util.CodingDecoding;

/**
 * Implements a text encoder.<br>
 * Actual encoding is actually done by a utility class : see {@link CodingDecoding#stringToBytes(String)}
 * @author Beyler Olivier FranceTelecom R&D
 */
public class TextEncoder implements IEncoder {
    /** Logger for this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            TextEncoder.class.getName());

    /** Default constructor. */
    public TextEncoder() {
        super();
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "create TextEncoder");
        }
    }

    /**
     * Encode.
     * @param object the String to be encoded
     * @return the byte[] form of the provided String
     * @see IEncoder#encode(Object)
     */
    public byte[] encode(Object object) {
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "encoding msg");
        }
        byte[] result = null;
        try {
            result = CodingDecoding.stringToBytes(object.toString());
        } catch (UnsupportedEncodingException e) {
            LOGGER.log(BasicLevel.ERROR, "encoding exception");
        }
        return result;
    }

    /**
     * Gets the handled object type.
     * @return the type (java.lang.String)
     * @see IEncoder#getType()
     */
    public String getType() {
        return String.class.getName();
    }
}
