package org.ow2.odis.engine;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.lifeCycle.engine.AbstractEngineState;
import org.ow2.odis.lifeCycle.engine.IEngineLifeCycle;
import org.ow2.odis.lifeCycle.engine.StateEngineLoaded;
import org.ow2.odis.node.Node;

/**
 * The base class for all ODIS engines.
 */
public abstract class AbstractEngine extends Thread implements IEngineLifeCycle {
    /** Logger for this class. */
    static final Logger LOGGER_ABSTRACT_ENGINE = Monolog.initialize()
            .getLogger(AbstractEngine.class.getName());

    /** The line separator constant. */
    static final String EOL = System.getProperty("line.separator");

    /** The ODIS node this engine belongs to. */
    protected final Node node;

    /** The state of this engine. */
    protected AbstractEngineState engineState = new StateEngineLoaded();

    /**
     * The state the engine is currently trying to reach (may not be relevant at all times).
     */
    protected AbstractEngineState wantedState = new StateEngineLoaded();

    /**
     * Standard operations for instantiating any ODIS node.
     * @param threadName the name of the thread that will run the engine
     * @param pNode the node the engine belongs to
     */
    public AbstractEngine(String threadName, Node pNode) {
        super(threadName);
        this.node = pNode;
    }

    /**
     * Gets the engine state.
     * @return AbstractEngineState
     * @see com.francetelecom.odis.component.lifeCycle.engine.IEngineLifeCycle#getEngineState()
     */
    public AbstractEngineState getEngineState() {
        return engineState;
    }

    /**
     * Sets the engine state.<br>
     * This is a plain setter. It does not trigger any other actions.
     * @param state the state
     */
    public void setEngineState(AbstractEngineState state) {
        engineState = state;
        if (LOGGER_ABSTRACT_ENGINE.isLoggable(BasicLevel.INFO)) {
            StringBuffer buffer = new StringBuffer(toString());
            buffer.append(" set to ");
            buffer.append(state.getStateName());
            LOGGER_ABSTRACT_ENGINE.log(BasicLevel.INFO, buffer.toString());
        }
    }

    /**
     * Gets the node the engine belongs to.
     * @return the node
     */
    public Node getNode() {
        return node;
    }
}
