package org.ow2.odis.lifeCycle;
/**
 * A cause can be set by an user request or a system request. A system request can be set to avoid a crash of the infrastructure. <br>
 * For exemple if the node is not able to send message, if it still produce new message the system may reach the limit of some buffer.
 */
public class Cause {
    /** The Constant BUFFER_OUT_OVERFLOW. */
    public static final int BUFFER_OUT_OVERFLOW = -1;

    /** The Constant SENDING_OVERFLOW. */
    public static final int SENDING_OVERFLOW = -2;

    /** The Constant LINK_ERROR. */
    public static final int LINK_ERROR = -2;

    /** The Constant USER_REQUEST. */
    public static final int USER_REQUEST = -3;

    /** The Constant EXTRA_DOMAIN_REQUEST. */
    public static final int EXTRA_DOMAIN_REQUEST = 10;

    /** The Constant DOMAIN_REQUEST. */
    public static final int DOMAIN_REQUEST = 20;

    /** The Constant NODE_REQUEST. */
    public static final int NODE_REQUEST = 30;

    /** The value. */
    private int value = 0;

    /** higher level of user witch set this cause. */
    private int userLevelRequest = 0;

    /** represent as a long the time when the cause start. */
    private long begin = -1;

    /** represent as a long when it finishes. */
    private long end = -1;

    /** contain informations about the cause. */
    private String detail = "";

    /**
     * Constructor.
     * @param value int value representing the cause
     */
    public Cause(final int value) {
        super();
        begin = System.currentTimeMillis();
        this.value = value;
        detail = "";
    }

    /**
     * Constructor.
     * @param value int value representing the cause
     * @param detail explication about the cause
     */
    public Cause(final int value, final String detail) {
        super();
        begin = System.currentTimeMillis();
        this.value = value;
        this.detail = detail;
    }

    /**
     * getter.
     * @return detail of this cause
     */
    public String getDetail() {
        return detail;
    }

    /**
     * setter of this cause.
     * @param detail of this cause.
     */
    public void setDetail(final String detail) {
        this.detail = detail;
    }

    /**
     * getter.
     * @return end time
     */
    public long getEnd() {
        return end;
    }

    /**
     * set the end whith the current time value.
     */
    public void setEnd() {
        this.end = System.currentTimeMillis();
    }

    /**
     * getter.
     * @return user level request.
     */
    public int getUserLevelRequest() {
        return userLevelRequest;
    }

    /**
     * setter.
     * @param userLevelRequest describe at with insfrastructure level the user set cause.
     */
    public void setUserLevelRequest(final int userLevelRequest) {
        this.userLevelRequest = userLevelRequest;
    }

    /**
     * getter.
     * @return the int value representing a specific cause
     */
    public int getValue() {
        return value;
    }

    /**
     * getter.
     * @return begin time
     */
    public long getBegin() {
        return begin;
    }
}
