package org.ow2.odis.lifeCycle.connection;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisConnectionException;
import org.ow2.odis.model.AbstractConnectionAttribute;

/**
 * base class of each xxxCnxState.
 * @author Beyler Olivier FranceTelecom R&D
 */
public abstract class AbstractConnectionState {
    /** logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(AbstractConnectionState.class.getName());

    /** represent the state. */
    protected int state;

    /**
     * Gets the state.
     * @return the state
     */
    public final int getState() {
        return state;
    }

    /**
     * Gets the state name.
     * @return a String representing the state.
     */
    public final String getStateName() {
        return IConnectionLifeCycle.STATES_NAME[getState()];
    }

    /**
     * define the next state aviable.
     * @param cnx current state.
     * @param wantedState next wanted state
     * @throws OdisConnectionException exception
     */
    public abstract void setLifeState(IConnectionLifeCycle cnx, AbstractConnectionState wantedState) throws OdisConnectionException;

    /**
     * define the next state avaiable.
     * @param cnx current state.
     * @param wantedState next wanted state
     * @param attribute give an attribute to the next state
     * @throws OdisConnectionException exception
     */
    public void setLifeState(final IConnectionLifeCycle cnx, final AbstractConnectionState wantedState, final AbstractConnectionAttribute attribute)
            throws OdisConnectionException {
        if (wantedState.state != state) {
            invalidTraitement(cnx, wantedState);
        }
    }

    /**
     * utility function to be able to format the error for logging and tracing 
     * exception.
     * @param cnx last connection state
     * @param wantedState wanted state
     * @throws OdisConnectionException exception
     */
    public void invalidTraitement(final IConnectionLifeCycle cnx, final AbstractConnectionState wantedState) throws OdisConnectionException {
        StringBuffer buff = new StringBuffer(cnx.getClass().getName());
        if (cnx.getAttribute()!=null){
            buff.append(" \"");
            buff.append(cnx.getAttribute().getName());
            buff.append("\"");
        }
        buff.append("  can not be set to ");
        buff.append(IConnectionLifeCycle.STATES_NAME[wantedState.getState()]);
        buff.append(" from ");
        buff.append(IConnectionLifeCycle.STATES_NAME[cnx.getConnectionState().getState()]);
        LOGGER.log(BasicLevel.ERROR, buff.toString());
        throw new OdisConnectionException(buff.toString());
    }
}
