package org.ow2.odis.lifeCycle.connection;

import org.ow2.odis.exception.OdisConnectionException;
import org.ow2.odis.model.AbstractConnectionAttribute;

/**
 * The Interface ICnxLifeCycle.
 * @author Beyler Olivier FranceTelecom R&D
 */
public interface IConnectionLifeCycle {
    /** The STATE LOADED. */
    int STATE_LOADED = 0;

    /** The STATE INITIALIZED. */
    int STATE_INITIALIZED = STATE_LOADED + 1;

    /** The STATE STARTED. */
    int STATE_STARTED = STATE_INITIALIZED + 1;

    /** The STATE STOPPED. */
    int STATE_STOPPED = STATE_STARTED + 1;

    /** The STATE UNLOADED. */
    int STATE_UNLOADED = STATE_STOPPED + 1;

    /** The STATES NAME. */
    String[] STATES_NAME = new String[] { "LOADED", "INITIALIZED", "STARTED", "STOPPED", "UNLOADED" };

    /**
     * Gets the cnx state.
     * @return state
     */
    AbstractConnectionState getConnectionState();

    /**
     * setter of the state.
     * @param state AbstractCnxState
     */
    void setConnectionState(AbstractConnectionState state);

    /**
     * Inits the component.
     * @throws OdisConnectionException the odis cnx exception
     */
    void initComponent() throws OdisConnectionException;

    /**
     * Launch component.
     * @throws OdisConnectionException the odis cnx exception
     */
    void launchComponent() throws OdisConnectionException;

    /**
     * Stop component.
     */
    void stopComponent();

    /**
     * Unload component.
     */
    void unloadComponent();

    /**
     * getter.
     * @return AbstractCnxAttribut
     */
    AbstractConnectionAttribute getAttribute();
}
