package org.ow2.odis.lifeCycle.connection;

import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.exception.OdisConnectionException;

/**
 * The validity of the parameters is confirmed. <br>
 * <br>
 * <b>on Socket Connection :</b>
 * <li>protocolIn /protocolOut is valid
 * <li>int parameters conversion froms string are correct.
 * <li>Server Socket position is valid. <br>
 * <br>
 * <b>on Memory connection :</b>
 * <li>int parameters conversion from string are correct. <br>
 * <br>
 * <b>on JMS Connection :</b>
 * <li>int parameters conversion from string are correct.
 * <li>cnx,session,queue/topic are well defined <br>
 * <img src="doc-files/statechart_connexion.png"/> <br>
 * <br>
 * @author Beyler Olivier FranceTelecom R&D
 */
public final class StateConnectionInitialized extends AbstractConnectionState {
    /**
     * Instantiates a new state cnx initialized.
     */
    public StateConnectionInitialized() {
        super();
        state = IConnectionLifeCycle.STATE_INITIALIZED;
    }

    /**
     * Sets the life state.
     * @param cnx the cnx
     * @param wantedState the wanted state
     * @throws OdisConnectionException the odis cnx exception
     * @see AbstractConnectionState#setLifeState(IConnectionLifeCycle,AbstractConnectionState)
     */
    public void setLifeState(final IConnectionLifeCycle cnx, final AbstractConnectionState wantedState) throws OdisConnectionException {
        StringBuffer buff;
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            buff = new StringBuffer(cnx.getClass().getName());
            buff.append(" setState to ");
            buff.append(IConnectionLifeCycle.STATES_NAME[wantedState.getState()]);
            LOGGER.log(BasicLevel.DEBUG, buff.toString());
        }
        switch (wantedState.getState()) {
        case IConnectionLifeCycle.STATE_INITIALIZED:
            // nothing to do
            break;
        case IConnectionLifeCycle.STATE_STARTED:
            try {
                cnx.launchComponent();
                cnx.setConnectionState(wantedState);
            } catch (OdisConnectionException e) {
                buff = new StringBuffer(cnx.getClass().getName());
                buff.append("Unable to launch component");
                LOGGER.log(BasicLevel.ERROR, buff.toString());
                cnx.stopComponent();
                cnx.setConnectionState(new StateConnectionStopped());
                throw e;
            }
            break;
        case IConnectionLifeCycle.STATE_STOPPED:
            cnx.stopComponent();
            cnx.setConnectionState(wantedState);
            break;
        default:
            invalidTraitement(cnx, wantedState);
        }
    }
}
