package org.ow2.odis.lifeCycle.connection;

import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.exception.OdisConnectionException;
import org.ow2.odis.jmx.JMX;
import org.ow2.odis.model.AbstractConnectionAttribute;

/**
 * <img src="doc-files/statechart_connexion.png"/>
 * @author Beyler Olivier FranceTelecom R&D
 */
public final class StateConnectionStopped extends AbstractConnectionState {
    /**
     * Instantiates a new state cnx stopped.
     */
    public StateConnectionStopped() {
        super();
        state = IConnectionLifeCycle.STATE_STOPPED;
    }

    /**
     * Sets the life state.
     * @param cnx the cnx
     * @param wantedState the wanted state
     * @throws OdisConnectionException the odis cnx exception
     * @see AbstractConnectionState#setLifeState(IConnectionLifeCycle,AbstractConnectionState)
     */
    public void setLifeState(final IConnectionLifeCycle cnx, final AbstractConnectionState wantedState) throws OdisConnectionException {
        StringBuffer buff;
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            buff = new StringBuffer(cnx.getClass().getName());
            buff.append(" setState to ");
            buff.append(IConnectionLifeCycle.STATES_NAME[wantedState.getState()]);
            LOGGER.log(BasicLevel.DEBUG, buff.toString());
        }
        switch (wantedState.getState()) {
        case IConnectionLifeCycle.STATE_STOPPED:
            // do nothing
            break;
        case IConnectionLifeCycle.STATE_UNLOADED:
            cnx.unloadComponent();
            cnx.setConnectionState(wantedState);
            AbstractConnectionAttribute attribute = cnx.getAttribute();
            if (attribute.getRegisterCount() == 1) {
                JMX.getInstance().unregisterMBean(attribute.getName(), attribute);
            }
            attribute.decreaseRegisterCount();
            break;
        default:
            invalidTraitement(cnx, wantedState);
        }
    }
}
