package org.ow2.odis.lifeCycle.domain;

import org.jdom.Element;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisDomainException;

/**
 * Base class of each domain state. <br>
 * Conception note : <i>Uses State design Pattern.</i> <br>
 * <img src="doc-files/DomainLifeCycle.png"/> <br>
 * <br>
 * @author Beyler Olivier FranceTelecom R&D
 */
public abstract class AbstractDomainState {
    /** logger of this class. */
    final static Logger LOGGER = Monolog.initialize().getLogger(AbstractDomainState.class.getName());

    /** int value representing the identity of this state. */
    protected int state;

    /**
     * Gets the state.
     * @return the state
     */
    public final int getState() {
        return state;
    }

    /**
     * a String representing the state.
     * @return state name
     */
    public final String getStateName() {
        return IDomainLifeCycle.STATES_NAME[getState()];
    }

    /**
     * Sets the life state.
     * @param domain the domain
     * @param wantedState the wanted state
     * @throws com.francetelecom.odis.exception.OdisDomainException *
     */
    public abstract void setLifeState(final IDomainLifeCycle domain, final AbstractDomainState wantedState) throws OdisDomainException;

    /**
     * Sets the life state.
     * @param domain the domain
     * @param wantedState the wanted state
     * @param attribute the attribut
     * @throws OdisDomainException
     */
    public void setLifeState(final IDomainLifeCycle domain, final AbstractDomainState wantedState, final Element attribute)
            throws OdisDomainException {
        StringBuffer buffer = new StringBuffer("The domain state ");
        buffer.append(getStateName());
        buffer.append(" doesn't accept setLifeState with 3 parameters");
        LOGGER.log(BasicLevel.FATAL, buffer.toString());
        invalidTreatment(domain, wantedState);
    }

    /**
     * Invalid treatment.
     * @param domain the domain
     * @param wantedState the wanted state
     * @throws com.francetelecom.odis.exception.OdisDomainException *
     */
    public void invalidTreatment(final IDomainLifeCycle domain, final AbstractDomainState wantedState) throws OdisDomainException {
        StringBuffer buff = new StringBuffer(domain.getClass().getName());
        buff.append("  can not be set to ");
        buff.append(IDomainLifeCycle.STATES_NAME[wantedState.getState()]);
        buff.append(" from ");
        buff.append(IDomainLifeCycle.STATES_NAME[domain.getDomainState().getState()]);
        LOGGER.log(BasicLevel.ERROR, buff.toString());
        throw new OdisDomainException(buff.toString());
    }
}
