package org.ow2.odis.lifeCycle.domain;

import org.jdom.Element;
import org.ow2.odis.exception.OdisDomainException;

/**
 * define the domaine life cycle state.
 * @author Beyler Olivier FranceTelecom R&D
 */
public interface IDomainLifeCycle {
    /** The STATE INITIAL. */
    int STATE_INITIAL = 0;

    /** The STATE LOADED. */
    int STATE_LOADED = 1;

    /** The STATE INITIALIZED. */
    int STATE_INITIALIZED = 2;

    /** The STATE STARTED. */
    int STATE_STARTED = 3;

    /** The STATE STOPPED. */
    int STATE_STOPPED = 4;

    /** The STATE UNLOADED. */
    int STATE_UNLOADED = 5;

    /** The state name. */
    String[] STATES_NAME = new String[] { "INITIAL", "LOADED", "INITIALIZED", "STARTED", "STOPPED", "UNLOADED" };

    /**
     * Gets the domain state.
     * @return the domain state
     */
    AbstractDomainState getDomainState();

    /**
     * Sets the domain state. *
     * @param state the new domain state
     */
    void setDomainState(AbstractDomainState state);

    /**
     * Inits the component.
     * @throws OdisDomainException the odis domain exception
     */
    void initComponent() throws OdisDomainException;

    /**
     * Launch component.
     * @throws OdisDomainException the odis domain exception
     */
    void launchComponent() throws OdisDomainException;

    /**
     * Stop component.
     * @throws OdisDomainException the odis domain exception
     */
    void stopComponent() throws OdisDomainException;

    /**
     * Unload component.
     * @throws OdisDomainException the odis domain exception
     */
    void unloadComponent() throws OdisDomainException;

    /**
     * Creates the attribut.
     * @param element the element
     * @throws OdisDomainException the odis domain exception
     */
    void createAttribut(Element element) throws OdisDomainException;
}
