package org.ow2.odis.lifeCycle.domain;

import org.jdom.Element;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.exception.OdisDomainException;

/**
 * Define the behaviors of Domain Initial. <br>
 * Conception note : <i>Uses State design Pattern.</i> <br>
 * <img src="doc-files/DomainLifeCycle.png"/> <br>
 * <br>
 * Copyright France Telecom 2005, All Rights Reserved.
 * @author Beyler Olivier FranceTelecom R&D
 */
public final class StateDomainInitial extends AbstractDomainState {
    /**
     * Instantiates a new state domain initial.
     */
    public StateDomainInitial() {
        super();
        state = IDomainLifeCycle.STATE_INITIAL;
    }

    /**
     * Sets the life state.
     * @param domain the domain
     * @param wantedState the wanted state
     * @throws OdisDomainException the odis domain exception
     * @see com.francetelecom.odis.component.lifeCycle.domain.AbstractDomainState#setLifeState(com.francetelecom.odis.component.lifeCycle.domain.IDomainLifeCycle,
     * com.francetelecom.odis.component.lifeCycle.domain.AbstractDomainState)
     */
    public void setLifeState(final IDomainLifeCycle domain, final AbstractDomainState wantedState) throws OdisDomainException {
        StringBuffer buff;
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            buff = new StringBuffer(domain.getClass().getName());
            buff.append(" setState to ");
            buff.append(IDomainLifeCycle.STATES_NAME[wantedState.getState()]);
            LOGGER.log(BasicLevel.DEBUG, buff.toString());
        }
        switch (wantedState.getState()) {
        case IDomainLifeCycle.STATE_INITIAL:
            // nothing to do
            break;
        case IDomainLifeCycle.STATE_STOPPED:
            domain.setDomainState(wantedState);
            break;
        case IDomainLifeCycle.STATE_UNLOADED:
            domain.unloadComponent();
            domain.setDomainState(wantedState);
            break;
        default:
            invalidTreatment(domain, wantedState);
        }
    }

    /**
     * Sets the life state.
     * @param domain the domain
     * @param wantedState the wanted state
     * @param element the element
     * @throws OdisDomainException the odis domain exception
     * @see AbstractDomainState#setLifeState(IDomainLifeCycle,AbstractDomainState,org.jdom.Element)
     */
    public void setLifeState(final IDomainLifeCycle domain, final AbstractDomainState wantedState, final Element element)
            throws OdisDomainException {
        StringBuffer buff;
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            buff = new StringBuffer(domain.getClass().getName());
            buff.append(" setState to ");
            buff.append(IDomainLifeCycle.STATES_NAME[wantedState.getState()]);
            LOGGER.log(BasicLevel.DEBUG, buff.toString());
        }
        if (wantedState.getState() == IDomainLifeCycle.STATE_LOADED) {
            domain.createAttribut(element);
            domain.setDomainState(wantedState);
        } else {
            invalidTreatment(domain, wantedState);
        }
    }
}
