package org.ow2.odis.lifeCycle.domain;

import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.exception.OdisDomainException;

/**
 * Define the behaviors of Domain Started. <br>
 * Conception note : <i>Uses State design Pattern.</i> <br>
 * <img src="doc-files/DomainLifeCycle.png"/> <br>
 * <br>
 * @author Beyler Olivier FranceTelecom R&D
 */
public final class StateDomainStarted extends AbstractDomainState {
    /**
     * Instantiates a new state domain started.
     */
    public StateDomainStarted() {
        super();
        state = IDomainLifeCycle.STATE_STARTED;
    }

    /**
     * Sets the life state.
     * @param domain the domain
     * @param wantedState the wanted state
     * @throws OdisDomainException the odis domain exception
     * @see AbstractDomainState#setLifeState(IDomainLifeCycle,AbstractDomainState)
     */
    public void setLifeState(final IDomainLifeCycle domain, final AbstractDomainState wantedState) throws OdisDomainException {
        StringBuffer buff;
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            buff = new StringBuffer(domain.getClass().getName());
            buff.append(" setState to ");
            buff.append(IDomainLifeCycle.STATES_NAME[wantedState.getState()]);
            LOGGER.log(BasicLevel.DEBUG, buff.toString());
        }
        switch (wantedState.getState()) {
        case IDomainLifeCycle.STATE_STARTED:
            // do nothing
            break;
        case IDomainLifeCycle.STATE_STOPPED:
            domain.stopComponent();
            domain.setDomainState(wantedState);
            break;
        default:
            invalidTreatment(domain, wantedState);
        }
    }
}
