package org.ow2.odis.lifeCycle.engine;

import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.exception.OdisEngineException;

/**
 * Define the behaviors of an Engine in LOADED state. <i>Follows State design Pattern.</i><br>
 * <img src="doc-files/EngineLifecycle.png"/>
 * @author Beyler Olivier FranceTelecom R&D
 */
public final class StateEngineLoaded extends AbstractEngineState {
    /**
     * Instantiates a new LOADED state.
     */
    public StateEngineLoaded() {
        super();
        state = IEngineLifeCycle.STATE_LOADED;
    }

    /**
     * Transition method.<br>
     * A loaded engine can be initialized or stopped.
     * @param engine the engine
     * @param wantedState the wanted state
     * @throws OdisEngineException if the transition is not allowed or the actions fail
     * @see AbstractEngineState#setLifeState(IEngineLifeCycle,AbstractEngineState)
     */
    public void setLifeState(final IEngineLifeCycle engine,
            final AbstractEngineState wantedState) throws OdisEngineException {
        StringBuffer buff;
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            buff = new StringBuffer(engine.getClass().getName());
            buff.append(" setLifeState to ");
            buff.append(wantedState.getStateName());
            LOGGER.log(BasicLevel.DEBUG, buff.toString());
        }
        switch (wantedState.getState()) {
        case IEngineLifeCycle.STATE_LOADED:
            // this is the current state : do nothing
            break;
        case IEngineLifeCycle.STATE_INITIALIZED:
            try {
                engine.initComponent();
                engine.setEngineState(wantedState);
            } catch (OdisEngineException e) {
                LOGGER.log(BasicLevel.ERROR, "Unable to initialize the engine");
                engine.setEngineState(new StateEngineStopped());
                throw e;
            }
            break;
        case IEngineLifeCycle.STATE_STOPPED:
            engine.setEngineState(wantedState);
            break;
        default:
            invalidTreatment(engine, wantedState);
        }
    }
}
