package org.ow2.odis.lifeCycle.node;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisNodeException;
import org.ow2.odis.model.NodeAttribute;

/**
 * Base class of Node State. <img src="doc-files/NodeLifeCycle.png"/>
 * @author Beyler Olivier FranceTelecom R&D
 */
public abstract class AbstractNodeState {
    /** logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(AbstractNodeState.class.getName());

    /** int value representing the identity of this state. */
    protected int state;

    /**
     * Gets the state.
     * @return the state
     */
    public final int getState() {
        return state;
    }

    /**
     * getter.
     * @return a String representing the state
     */
    public final String getStateName() {
        return INodeLifeCycle.STATES_NAME[getState()];
    }

    /**
     * Sets the life state.
     * @param node INodeLifeCycle
     * @param wantedState AbstractNodeState
     * @throws OdisNodeException the odis node exception
     */
    public abstract void setLifeState(INodeLifeCycle node, AbstractNodeState wantedState) throws OdisNodeException;

    /**
     * Sets the life state.
     * @param node INodeLifeCycle
     * @param wantedState AbstractNodeState
     * @param attribute the attribute
     * @throws OdisNodeException the odis node exception
     */
    public void setLifeState(final INodeLifeCycle node, final AbstractNodeState wantedState, final NodeAttribute attribute)
            throws OdisNodeException {
        invalidTreatment(node, wantedState);
    }

    /**
     * Invalid treatment.
     * @param node INodeLifeCycle
     * @param wantedState AbstractNodeState
     * @throws OdisNodeException the odis node exception
     */
    public void invalidTreatment(final INodeLifeCycle node, final AbstractNodeState wantedState) throws OdisNodeException {
        StringBuffer buff = new StringBuffer(node.getClass().getName());
        buff.append('(');
        buff.append(node.toString());
        buff.append(')');
        buff.append(" can not be set to ");
        buff.append(INodeLifeCycle.STATES_NAME[wantedState.getState()]);
        buff.append(" from ");
        buff.append(INodeLifeCycle.STATES_NAME[node.getNodeState().getState()]);
        LOGGER.log(BasicLevel.ERROR, buff.toString());
        throw new OdisNodeException(buff.toString());
    }
}
