package org.ow2.odis.lifeCycle.node;

import org.ow2.odis.engine.EngineIn;
import org.ow2.odis.engine.EngineOut;
import org.ow2.odis.engine.EngineProceed;
import org.ow2.odis.exception.OdisNodeException;

/**
 * The Interface INodeLifeCycle.
 * @author Beyler Olivier FranceTelecom R&D
 */
public interface INodeLifeCycle {
    /** The STATE LOADED. */
    int STATE_LOADED = 0;

    /** The STATE INITIALIZED. */
    int STATE_INITIALIZED = STATE_LOADED + 1; //1

    /** The STATE STARTED. */
    int STATE_STARTED = STATE_INITIALIZED + 1; //2

    /** The STATE SUSPENDED. */
    int STATE_SUSPENDED = STATE_STARTED + 1; //3

    /** The STATE STOPPED. */
    int STATE_STOPPED = STATE_SUSPENDED + 1; //4

    /** The STATE UNLOADED. */
    int STATE_UNLOADED = STATE_STOPPED + 1; //5

    /** The state name. */
    String[] STATES_NAME = new String[] { "LOADED", "INITIALIZED", "STARTED", "SUSPENDED", "STOPPED", "UNLOADED" };

    /**
     * Gets the node state.
     * @return the node state
     */
    AbstractNodeState getNodeState();

    /**
     * Sets the node state. *
     * @param state the new node state
     */
    void setNodeState(AbstractNodeState state);

    /**
     * Inits the component.
     * @throws OdisNodeException the odis node exception
     */
    void initComponent() throws OdisNodeException;

    /**
     * Launch component.
     * @throws OdisNodeException the odis node exception
     */
    void launchComponent() throws OdisNodeException;

    /**
     * Suspend component.
     */
    void suspendComponent();

    /**
     * Stop component.
     * @throws OdisNodeException the odis node exception
     */
    void stopComponent() throws OdisNodeException;

    /**
     * Unload component.
     */
    void unloadComponent();

    /**
     * Gets the engine in.
     * @return the engine in
     */
    EngineIn getEngineIn();

    /**
     * Gets the engine out.
     * @return the engine out
     */
    EngineOut getEngineOut();

    /**
     * Gets the engine proceed.
     * @return the engine proceed
     */
    EngineProceed getEngineProceed();
}
