package org.ow2.odis.lifeCycle.node;

import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.exception.OdisNodeException;

/**
 * Define the behaviors of Node Loaded. <i>Uses State design Pattern.</i> <br>
 * <img src="doc-files/NodeLifeCycle.png"/>
 * @author Beyler Olivier FranceTelecom R&D
 */
public final class StateNodeLoaded extends AbstractNodeState {
    /**
     * Instantiates a new state node loaded.
     */
    public StateNodeLoaded() {
        super();
        state = INodeLifeCycle.STATE_LOADED;
    }

    /**
     * Sets the life state.
     * @param node the node
     * @param wantedState the wanted state
     * @throws OdisNodeException the odis node exception
     * @see AbstractNodeState#setLifeState(INodeLifeCycle,AbstractNodeState)
     */
    public void setLifeState(final INodeLifeCycle node, final AbstractNodeState wantedState) throws OdisNodeException {
        StringBuffer buff;
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            buff = new StringBuffer(node.getClass().getName());
            buff.append(" setState to ");
            buff.append(INodeLifeCycle.STATES_NAME[wantedState.getState()]);
            LOGGER.log(BasicLevel.DEBUG, buff.toString());
        }
        switch (wantedState.getState()) {
        case INodeLifeCycle.STATE_LOADED:
            // nothing to do
            break;
        case INodeLifeCycle.STATE_INITIALIZED:
            node.initComponent();
            node.setNodeState(wantedState);
            break;
        case INodeLifeCycle.STATE_STOPPED:
            node.stopComponent();
            node.setNodeState(wantedState);
            break;
        case INodeLifeCycle.STATE_UNLOADED:
            node.unloadComponent();
            node.setNodeState(wantedState);
            break;
        default:
            invalidTreatment(node, wantedState);
        }
    }
}
