package org.ow2.odis.model;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jdom.Element;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.exception.OdisAttributeException;

/**
 * define properties of a domain.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class DomainAttribute extends AbstractAttribute {
    /**
     * Instantiates a new domain attribut.
     * @param parent the parent
     */
    public DomainAttribute(AbstractAttribute parent) {
        // no parent for a domain.
        super(null);
    }

    // ** The Constant ADMIN. */
    // private static final String ADMIN = "ADMIN";
    /** boolean value witch indicate if the domain stop in case of error. */
    private boolean stopOnError = true;

    /** domain's name. */
    private String name = "";

    /** root path of the domain used by persistance. */
    private String path = "";

    /** contain list of node attribut. */
    public List lsNodeAttributs = new ArrayList();

    /** Contain list of Connection attribut. */
    public List lsCnxAttributs = new ArrayList();

    /**
     * Gets the name.
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * Sets the name. *
     * @param name the new name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Gets the path. It defines the base of path use by persistance.
     * @return the path
     */
    public String getPath() {
        return path;
    }

    /**
     * Sets the path. *
     * @param path the new path
     */
    public void setPath(String path) {
        this.path = path;
    }

    /**
     * Write XML.
     * @param out the out
     * @param full the full
     * @throws IOException Signals that an I/O exception has occurred.
     * @see com.AbstractAttribute.odis.model.AbstractAttribut#writeXML(java.io.OutputStream, boolean)
     */
    public void writeXML(OutputStream out, boolean full) throws IOException {
        out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n".getBytes());
        out
                .write(("<odis xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"ODISXMLSchema3_1.xsd\">" + Const.EOL)
                        .getBytes());
        out.write((Const.EOL).getBytes());
        out.write((Const.TAB + "<DOMAIN>" + Const.EOL).getBytes());
        out
                .write((Const.TAB + Const.TAB + "<NAME value =\"" + name
                        + "\"/>" + Const.EOL).getBytes());
        out
                .write((Const.TAB + Const.TAB + "<PATH value =\"" + path
                        + "\"/>" + Const.EOL).getBytes());
        out.write((Const.TAB + "</DOMAIN>" + Const.EOL).getBytes());
        AbstractAttribute attribute;
        out.write((Const.EOL + Const.nodeSep).getBytes());
        out.write((Const.EOL + "<NODES>").getBytes());
        Iterator itNodeAttributs = lsNodeAttributs.iterator();
        while (itNodeAttributs.hasNext()) {
            attribute = (AbstractAttribute) itNodeAttributs.next();
            out.write((Const.EOL).getBytes());
            attribute.writeXML(out, full);
        }
        out.write((Const.EOL + "</NODES>").getBytes());
        out.write((Const.EOL + Const.cnxSep).getBytes());
        out.write((Const.EOL + "<CONNECTIONS>").getBytes());
        Iterator itCnxAttribut = lsCnxAttributs.iterator();
        while (itCnxAttribut.hasNext()) {
            attribute = (AbstractAttribute) itCnxAttribut.next();
            out.write((Const.EOL).getBytes());
            attribute.writeXML(out, full);
        }
        out.write((Const.EOL + "</CONNECTIONS>").getBytes());
        out.write("</odis>\n".getBytes());
    }

    /**
     * Check properties.
     * @throws OdisAttributeException the odis attribute exception
     */
    public void checkProperties() throws OdisAttributeException {
        // nodes recherche doublon
        Map map = new Hashtable();
        String nameNode;
        NodeAttribute nodeAttribut;
        Set lstoNodeRemoves = new HashSet();
        Iterator iterator = lsNodeAttributs.iterator();
        while (iterator.hasNext()) {
            nodeAttribut = (NodeAttribute) iterator.next();
            nameNode = nodeAttribut.getName();
            if (!map.containsKey(nameNode)) {
                map.put(nameNode, nodeAttribut);
            } else {
                lstoNodeRemoves.add(nodeAttribut);
                lstoNodeRemoves.add(map.get(nameNode));
            }
        }
        // connections recherche doublon
        String nameCnx;
        map = new Hashtable();
        Set lstoCnxRemoves = new HashSet();
        AbstractConnectionAttribute cnxAttribut;
        iterator = lsCnxAttributs.iterator();
        while (iterator.hasNext()) {
            cnxAttribut = (AbstractConnectionAttribute) iterator.next();
            nameCnx = cnxAttribut.getName();
            if (!map.containsKey(nameCnx)) {
                map.put(nameCnx, cnxAttribut);
            } else {
                lstoCnxRemoves.add(cnxAttribut);
                lstoCnxRemoves.add(map.get(nameCnx));
            }
        }
        if (stopOnError) {
            // aucune erreur de configuration n'est tolere : on sort en
            // exception
            if (!lstoCnxRemoves.isEmpty() || !lstoNodeRemoves.isEmpty()) {
                map.clear();
                throw new OdisAttributeException(
                        "Duplicate Nodes or Connections: "
                                + lstoNodeRemoves.toString()
                                + lstoCnxRemoves.toString());
            }
        } else {
            // removes all duplicate node
            NodeAttribute attribute;
            iterator = lstoNodeRemoves.iterator();
            if (!lstoNodeRemoves.isEmpty()) {
                StringBuffer buffer = new StringBuffer(
                        "Remove duplicate nodes :");
                while (iterator.hasNext()) {
                    attribute = (NodeAttribute) iterator.next();
                    buffer.append(attribute.getName());
                    buffer.append(" ");
                    lsNodeAttributs.remove(attribute);
                }
                LOGGER.log(BasicLevel.ERROR, buffer.toString());
            }
            // removes all duplicate Cnx
            AbstractConnectionAttribute attributCnx;
            iterator = lstoCnxRemoves.iterator();
            if (!lstoNodeRemoves.isEmpty()) {
                StringBuffer buffer = new StringBuffer(
                        "Remove duplicate Connection:");
                while (iterator.hasNext()) {
                    attributCnx = (AbstractConnectionAttribute) iterator.next();
                    buffer.append(attributCnx.getName());
                    buffer.append(" ");
                    lsCnxAttributs.remove(attributCnx);
                }
                LOGGER.log(BasicLevel.ERROR, buffer.toString());
            }
        }
        map.clear();
        iterator = lsNodeAttributs.iterator();
        while (iterator.hasNext()) {
            nodeAttribut = (NodeAttribute) iterator.next();
            nodeAttribut.checkProperties();
        }
    }

    /**
     * stop on error in case of exception the domain is halted or not.
     * @return boolean
     */
    public boolean isStopOnError() {
        return stopOnError;
    }

    /**
     * Sets the stop on error.
     * @param stopOnError the new stop on error
     */
    public void setStopOnError(boolean stopOnError) {
        this.stopOnError = stopOnError;
    }

    /**
     * Gets the PName.
     * @return String
     * @see com.AbstractAttribute.odis.model.AbstractAttribut#getPName()
     */
    public String getPName() {
        StringBuffer buffer = new StringBuffer("Domain");
        buffer.append(Const.CST_SEP);
        buffer.append(getName());
        return buffer.toString();
    }

    /**
     * Set the element.
     * @param domainDeclaration Element
     */
    public void setter(Element domainDeclaration) {
        if (domainDeclaration == null) {
            LOGGER.log(BasicLevel.INFO, "<DOMAIN></DOMAIN> not defined");
        } else {
            Element domaineNameElt = domainDeclaration
                    .getChild(Const.DOMAIN_NAME);
            if (domaineNameElt != null) {
                name = domaineNameElt.getAttributeValue(Const.VALUE, name);
            }
            Element domainePathElt = domainDeclaration
                    .getChild(Const.DOMAIN_PATH);
            if (domainePathElt != null) {
                path = domainePathElt.getAttributeValue(Const.VALUE, path);
            }
        }
    }
}
