package org.ow2.odis.model;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import org.jdom.Element;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.connection.ConnectionFactory;
import org.ow2.odis.connection.socket.IProtocolIn;
import org.ow2.odis.connection.socket.IProtocolOut;
import org.ow2.odis.connection.socket.OdisProtocol;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.loader.GenericFactory;
import org.ow2.odis.util.CheckUtilities;

/**
 * Defines the attribute for a socket connection.
 */
public class SocketConnectionAttribute extends AbstractConnectionAttribute implements
        SocketConnectionAttributeMBean {
    /**
     * Instantiates a new socket cnx attribute.
     * @param parent the parent
     */
    public SocketConnectionAttribute(AbstractAttribute parent) {
        super(parent);
    }

    /** The port. */
    private int port = 123;

    /** The str port. */
    private String strPort = "123";

    /** The size max object. */
    private int sizeMaxObject = 5000;

    /** The str size max object. */
    private String strSizeMaxObject = "5000";

    /** The size byte max. */
    private int sizeByteMax = 100000;

    /** The str size byte max. */
    private String strSizeByteMax = "100000";

    /** The address. */
    private String address = "localhost";

    /** The protocol in. */
    private String protocolIn = OdisProtocol.class.getName();

    /** The protocol out. */
    private String protocolOut = OdisProtocol.class.getName();

    /** The protocol in version. */
    protected String protocolInVersion = GenericFactory.DEFAULT_VERSION;

    /** The protocol out version. */
    protected String protocolOutVersion = GenericFactory.DEFAULT_VERSION;

    /** The server side. */
    private String serverSide = "IN";

    /** The state. */
    private int state = Const.READY;

    /** The size byte. */
    private long sizeByte = 0;

    /**
     * equals.
     * @param obj to be compare
     * @return boolean
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        boolean result = false;
        SocketConnectionAttribute copy = null;
        try {
            copy = (SocketConnectionAttribute) obj;
        } catch (ClassCastException e) {
            return false;
        }
        result = super.equals(obj);
        if (result) {
            result &= ((address == null && copy.address == null) || copy.address
                    .equals(this.address));
            result &= ((protocolIn == null && copy.protocolIn == null) || copy.protocolIn
                    .equals(this.protocolIn));
            result &= ((protocolOut == null && copy.protocolOut == null) || copy.protocolOut
                    .equals(this.protocolOut));
            result &= ((serverSide == null && copy.serverSide == null) || copy.serverSide
                    .equals(this.serverSide));
            result &= ((strMinClient == null && copy.strMinClient == null) || copy.strMinClient
                    .equals(this.strMinClient));
            result &= ((strPort == null && copy.strPort == null) || copy.strPort
                    .equals(this.strPort));
            result &= ((strSizeByteMax == null && copy.strSizeByteMax == null) || copy.strSizeByteMax
                    .equals(this.strSizeByteMax));
            result &= ((strSizeMaxObject == null && copy.strSizeMaxObject == null) || copy.strSizeMaxObject
                    .equals(this.strSizeMaxObject));
            result &= ((strTimeOut == null && copy.strTimeOut == null) || copy.strTimeOut
                    .equals(this.strTimeOut));
            if (!result) {
                traceOnConfictualConnection(copy);
            }
        }
        return result;
    }

    /**
     * getter.
     * @return the size byte contain into socket connection
     */
    public long getSizeByte() {
        return sizeByte;
    }

    /**
     * increase (or decrease in case of negative param) the size of byte contain into the socket connection.
     * @param value value of increament. It can be negative.
     */
    public void incSizeByte(long value) {
        sizeByte += value;
    }

    /** The number message. */
    private int numberMessage = 0;

    /**
     * increase (or decrease in case of negative param) the number of message contain by the socket connection.
     * @param value value of increament. It can be negative.
     */
    public void incNumberMessage(int value) {
        numberMessage += value;
    }

    /** The number connection. */
    private int numberConnection = 0;

    /**
     * increase (or decrease in case of negative param) the number connection contain by the socket connection. This number represent both
     * client and server connectxion.
     * @param value value of increment. It can be negative.
     */
    public void incNumberConnection(int value) {
        numberConnection += value;
    }

    /** The time out. */
    private int timeOut = 0;

    /** The str time out. */
    private String strTimeOut = "0";

    /** The min client. */
    private int minClient = 0;

    /** The str min client. */
    private String strMinClient = "0";

    /**
     * Setter.
     * @param cnxDeclaration the declaration
     */
    public void subSetter(Element cnxDeclaration) {
        List lsUnusedProperties = cnxDeclaration.getChildren();
        Element element = cnxDeclaration.getChild(Const.PROPERTY_PORT);
        if (element != null) {
            strPort = element.getAttributeValue(Const.VALUE, strPort);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(Const.PROPERTY_ADRESS);
        if (element != null) {
            address = element.getAttributeValue(Const.VALUE, address);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(Const.PROPERTY_SERVER_SIDE);
        if (element != null) {
            serverSide = element.getAttributeValue(Const.VALUE, serverSide);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(Const.PROPERTY_TIMEOUT);
        if (element != null) {
            strTimeOut = element.getAttributeValue(Const.VALUE, strTimeOut);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(Const.PROPERTY_SIZE_BYTE_MAX);
        if (element != null) {
            strSizeByteMax = element.getAttributeValue(Const.VALUE,
                    strSizeByteMax);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(Const.PROPERTY_MIN_CLIENT);
        if (element != null) {
            strMinClient = element.getAttributeValue(Const.VALUE, strMinClient);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(Const.PROPERTY_SIZE_MAX_OBJECT);
        if (element != null) {
            strSizeMaxObject = element.getAttributeValue(Const.VALUE,
                    strSizeMaxObject);
            lsUnusedProperties.remove(element);
        }
        subSubSetter(cnxDeclaration, lsUnusedProperties);
        CheckUtilities.checkUnusedProperties(lsUnusedProperties, LOGGER);
    }

    /**
     * Sets the values for implementation dependent elements.<br>
     * For a standard SOCKET connection, these are ProtocolIn and ProtocolOut.
     * @param cnxDeclaration the connection declaration
     * @param lsUnusedProperties the list of unused properties, managed by subSetter
     */
    public void subSubSetter(Element cnxDeclaration, List lsUnusedProperties) {
        Element element = cnxDeclaration.getChild(Const.PROPERTY_PROTOCOL_IN);
        if (element != null) {
            protocolIn = element.getAttributeValue(Const.VALUE, protocolIn);
            protocolInVersion = element.getAttributeValue(Const.VERSION,
                    protocolInVersion);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(Const.PROPERTY_PROTOCOL_OUT);
        if (element != null) {
            protocolOut = element.getAttributeValue(Const.VALUE, protocolOut);
            protocolOutVersion = element.getAttributeValue(Const.VERSION,
                    protocolOutVersion);
            lsUnusedProperties.remove(element);
        }
    }

    /**
     * getter.
     * @return the type of this connection.
     */
    public String getTypeConnection() {
        return ConnectionFactory.CONNECTION_SOCKET;
    }

    /**
     * create a new ProtocoleIn according to the class name.
     * @return new IProtocoleIn instance
     * @throws OdisAttributeException can occurred if the class name is unknown or if this class doesn't inherit from IProtocoleIn
     */
    public IProtocolIn createProtocolInInstance() throws OdisAttributeException {
        IProtocolIn result = null;
        try {
            result = (IProtocolIn) CheckUtilities.createInstance(protocolIn,
                    protocolInVersion, "protocolIn", LOGGER);
        } catch (ClassCastException e) {
            StringBuffer buff = new StringBuffer("protocolIn class \"");
            buff.append(protocolIn);
            buff.append("\" is invalid");
            LOGGER.log(BasicLevel.ERROR, buff.toString());
            throw new OdisAttributeException(buff.toString(), e);
        }
        return result;
    }

    /**
     * create a new ProtocoleOut according to the class name.
     * @return new IProtocoleOut instance
     * @throws OdisAttributeException can occurred if the class name is unknown or if this class doesn't inherit from IProtocoleOut
     */
    public IProtocolOut createProtocolOutInstance()
            throws OdisAttributeException {
        IProtocolOut result = null;
        try {
            result = (IProtocolOut) CheckUtilities.createInstance(protocolOut,
                    protocolOutVersion, "protocolOut", LOGGER);
        } catch (ClassCastException e) {
            StringBuffer buff = new StringBuffer("protocolOut class \"");
            buff.append(protocolOut);
            buff.append("\" is invalid");
            LOGGER.log(BasicLevel.ERROR, buff.toString());
            throw new OdisAttributeException(buff.toString(), e);
        }
        return result;
    }

    /**
     * Checks the validity of all SocketCnxAttribut parameters.
     * @throws OdisAttributeException exception in case of error
     * @see com.AbstractAttribute.odis.model.AbstractAttribut#checkProperties()
     */
    public void checkProperties() throws OdisAttributeException {
        StringBuffer buff = null;
        try {
            setPort(Integer.parseInt(strPort));
        } catch (NumberFormatException e) {
            buff = concatException(buff, Const.PROPERTY_PORT, strPort, e);
        }
        try {
            setTimeOut(Integer.parseInt(strTimeOut));
        } catch (NumberFormatException e) {
            buff = concatException(buff, Const.PROPERTY_TIMEOUT, strTimeOut, e);
        }
        try {
            setSizeByteMax(Integer.parseInt(strSizeByteMax));
        } catch (NumberFormatException e) {
            buff = concatException(buff, Const.PROPERTY_SIZE_BYTE_MAX,
                    strSizeByteMax, e);
        }
        try {
            setMinClient(Integer.parseInt(strMinClient));
        } catch (NumberFormatException e) {
            buff = concatException(buff, Const.PROPERTY_MIN_CLIENT,
                    strMinClient, e);
        }
        try {
            setSizeObjectMax(Integer.parseInt(strSizeMaxObject));
        } catch (NumberFormatException e) {
            buff = concatException(buff, Const.PROPERTY_SIZE_MAX_OBJECT,
                    strSizeMaxObject, e);
        }
        if (!(serverSide.equals("IN") || serverSide.equals("OUT"))) {
            buff = concatException(buff, Const.PROPERTY_SERVER_SIDE,
                    serverSide, null);
            buff.append(Const.EOL);
            buff.append("Only authorized values are 'IN' or 'OUT'.");
        }
        if (buff != null) {
            LOGGER.log(BasicLevel.ERROR, buff.toString());
            throw new OdisAttributeException(buff.toString());
        }
    }

    /**
     * Checks the validity of implementation specific parameters.<br>
     * For a standard SOCKET connection, these are ProtocolIn and ProtocolOut.
     * @param buff a StringBuffer used to store the problems found.
     */
    public void subCheckProperties(StringBuffer buff) {
        try {
            createProtocolInInstance();
        } catch (OdisAttributeException e) {
            buff = concatException(buff, Const.PROPERTY_PROTOCOL_IN,
                    protocolIn, e);
        }
        try {
            createProtocolOutInstance();
        } catch (OdisAttributeException e) {
            buff = concatException(buff, Const.PROPERTY_PROTOCOL_OUT,
                    protocolOut, e);
        }
    }

    /**
     * getter.
     * @return port
     */
    public int getPort() {
        return port;
    }

    /**
     * setter.
     * @param pPort int
     */
    public void setPort(int pPort) {
        this.port = pPort;
    }

    /**
     * getter.
     * @return String class name of protocoleIn
     */
    public String getProtocolIn() {
        return protocolIn;
    }

    /**
     * setter.
     * @param pProtocolIn class name of protocoleIn
     */
    public void setProtocolIn(String pProtocolIn) {
        this.protocolIn = pProtocolIn;
    }

    /**
     * getter.
     * @return the class name use for the protocol Out
     */
    public String getProtocolOut() {
        return protocolOut;
    }

    /**
     * setter.
     * @param pProtocolOut the class name use for the protocol Out
     */
    public void setProtocolOut(String pProtocolOut) {
        this.protocolOut = pProtocolOut;
    }

    /**
     * getter.
     * @return the ip address of the connection.
     */
    public String getAddress() {
        return address;
    }

    /**
     * setter.
     * @param pAdress address IP of the connection.
     */
    public void setAddress(String pAdress) {
        this.address = pAdress;
    }

    /**
     * setter of server side witch determine where server is located.
     * @param side can be IN or OUT value
     */
    public void setServerSide(String side) {
        if (Const.PROPERTY_IN.equals(side)) {
            serverSide = Const.PROPERTY_IN;
        } else if (Const.PROPERTY_OUT.equals(side)) {
            serverSide = Const.PROPERTY_OUT;
        } else {
            LOGGER.log(BasicLevel.ERROR, "unknown value " + side + " for "
                    + Const.PROPERTY_SERVER_SIDE + " 's property");
        }
        this.serverSide = side;
    }

    /**
     * getter of server side.
     * @return the server location.
     */
    public String getServerSide() {
        return serverSide;
    }

    /**
     * getter.
     * @return the size max of object contain by SocketCnxAttribut
     */
    public int getSizeObjectMax() {
        return sizeMaxObject;
    }

    /**
     * setter.
     * @param sizeMax the size max of object contain by SocketCnxAttribut
     */
    public void setSizeObjectMax(int sizeMax) {
        this.sizeMaxObject = sizeMax;
    }

    /**
     * getter.
     * @return the size max of bytes contain by SocketCnxAttribut
     */
    public int getSizeByteMax() {
        return sizeByteMax;
    }

    /**
     * setter.
     * @param pValue the size max of bytes contain by SocketCnxAttribut
     */
    public void setSizeByteMax(int pValue) {
        this.sizeByteMax = pValue;
        setChanged();
        this.notifyObservers(this);
    }

    /**
     * Gets the state.
     * @return the state
     */
    public int getState() {
        return state;
    }

    /**
     * Sets the state. *
     * @param pState the new state
     */
    public void setState(int pState) {
        this.state = pState;
    }

    /**
     * getter.
     * @return string representing the state.
     */
    public String getStateString() {
        return state == Const.READY ? Const.STR_READY : Const.STR_NOT_READY;
    }

    /**
     * getter.
     * @return the number of message contained by the connection.
     */
    public int getNumberMessage() {
        return numberMessage;
    }

    /**
     * getter.
     * @return the number of connection done on a socket
     */
    public int getNumberConnection() {
        return numberConnection;
    }

    /**
     * getter.
     * @return the time out before the connection break if no response
     */
    public int getTimeOut() {
        return timeOut;
    }

    /**
     * setter.
     * @param pTimeOut the time out before the connection break if no response
     */
    public void setTimeOut(int pTimeOut) {
        this.timeOut = pTimeOut;
    }

    /**
     * Write XML.
     * @param out the out
     * @param full the full
     * @throws java.io.IOException *
     */
    public void writeXML(OutputStream out, boolean full) throws IOException {
        out
                .write((Const.TAB + "<SOCKET name=\"" + getName() + "\">" + Const.EOL)
                        .getBytes());
        writeXMLCNXUtilities(out, 2, Const.PROPERTY_SERVER_SIDE,
                getServerSide(), "", full);
        writeXMLCNXUtilities(out, 2, Const.PROPERTY_ADRESS, getAddress(),
                "localhost", full);
        writeXMLCNXUtilities(out, 2, Const.PROPERTY_PORT, "" + getPort(),
                "4000", full);
        writeXMLCNXUtilities(out, 2, Const.PROPERTY_PROTOCOL_IN,
                getProtocolIn(), OdisProtocol.class.getName(), full);
        writeXMLCNXUtilities(out, 2, Const.PROPERTY_PROTOCOL_OUT,
                getProtocolOut(), OdisProtocol.class.getName(), full);
        out.write((Const.TAB + "</SOCKET>" + Const.EOL).getBytes());
    }

    /**
     * getter.
     * @return the minimum number of client requested before considering that the send is correct.
     */
    public int getMinClient() {
        return minClient;
    }

    /**
     * setter.
     * @param pMinClient : the minimum number of client requested before considering that the send is correct.
     */
    public void setMinClient(final int pMinClient) {
        this.minClient = pMinClient;
    }

    /**
     * Gets the P name.
     * @return the p name
     * @see com.AbstractAttribute.odis.model.AbstractAttribut#getPName()
     */
    public String getPName() {
        StringBuffer buffer = new StringBuffer(getParent().getPName());
        buffer.append(Const.CST_SEP);
        buffer.append("SocketCnx");
        return buffer.toString();
    }
}
