package org.ow2.odis.model;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import org.jdom.Element;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.port.PortOut;
import org.ow2.odis.util.CheckUtilities;

/**
 * The Class ToAttribut.
 * @author Nicolas Tachker
 */
public class ToAttribute extends AbstractAttribute {
    /**
     * Instantiates a new to attribute.
     * @param parent the parent
     */
    public ToAttribute(AbstractAttribute parent) {
        super(parent);
    }

    /** The str type. */
    private String strType = null;

    /** The routing. */
    private String routing = null;

    /** The link. */
    private String link = null;

    /** The port out. */
    private PortOut portOut = null;

    /**
     * Setter.
     * @param declaration Element
     */
    public void setter(Element declaration) {
        if (declaration != null) {
            List lsUnusedProperties = declaration.getChildren();
            link = declaration.getAttributeValue(Const.CST_LINK);
            Element element = declaration.getChild(Const.CST_TYPE);
            if (element != null) {
                strType = element.getAttributeValue(Const.VALUE, strType);
                lsUnusedProperties.remove(element);
            }
            element = declaration.getChild(Const.CST_ROUTING);
            if (element != null) {
                routing = element.getAttributeValue(Const.VALUE, routing);
                lsUnusedProperties.remove(element);
            }
            CheckUtilities.checkUnusedProperties(lsUnusedProperties, LOGGER);
        }
    }

    /**
     * Gets the id.
     * @return routing if defined or type if routing is not defined
     */
    public String getId() {
        String result;
        if ("".equals(getRouting()) || getRouting() == null) {
            result = strType;
        } else {
            result = getRouting();
        }
        return result;
    }

    /**
     * Gets the link.
     * @return the link
     */
    public String getLink() {
        return link;
    }

    /**
     * Gets the routing.
     * @return the routing
     */
    public String getRouting() {
        return routing;
    }

    /**
     * Gets the str type.
     * @return the str type
     */
    public String getStrType() {
        return strType;
    }

    /**
     * write XML.
     * @param out the out
     * @param full the full
     * @throws IOException Signals that an I/O exception has occurred.
     * @see com.AbstractAttribute.odis.model.AbstractAttribut#writeXML(java.io.OutputStream, boolean)
     */
    public void writeXML(OutputStream out, boolean full) throws IOException {
        out.write((Const.TAB2 + "<TO link=\"" + link + "\">" + Const.EOL)
                .getBytes());
        out
                .write((Const.TAB3 + "<TYPE value=\"" + strType + "\"/>" + Const.EOL)
                        .getBytes());
        writeXMLUtilities(out, 3, Const.CST_ROUTING, routing, "", false);
        out.write((Const.TAB2 + "</TO>" + Const.EOL).getBytes());
    }

    /**
     * Check properties.
     * @throws OdisAttributeException the odis attribute exception
     * @see AbstractAttribute#checkProperties()
     */
    public void checkProperties() throws OdisAttributeException {
        // nothing to do
    }

    /**
     * Gets the port out.
     * @return the port out
     */
    public PortOut getPortOut() {
        return portOut;
    }

    /**
     * Sets the port out. *
     * @param portOut the new port out
     */
    public void setPortOut(PortOut portOut) {
        this.portOut = portOut;
    }

    /**
     * Gets the P name.
     * @return String
     * @see com.AbstractAttribute.odis.model.AbstractAttribut#getPName()
     */
    public String getPName() {
        StringBuffer buffer = new StringBuffer(getParent().getPName());
        buffer.append(Const.CST_SEP);
        buffer.append("portOut_");
        buffer.append(getId());
        return buffer.toString();
    }

    /**
     * Compares the specified object with <i>this</i> for equality. Returns <tt>true</tt> if and only if the specified object is also a
     * ToAttribut and their discriminating parameters are <i>equal</i>.
     * @param obj the object to be compared for equality with <i>this</i>.
     * @return <tt>true</tt> if the specified object is equal to <i>this</i>.
     */
    public boolean equals(Object obj) {
        boolean result = false;
        ToAttribute copy = null;
        try {
            copy = (ToAttribute) obj;
        } catch (ClassCastException e) {
            return false;
        }
        result = copy.strType.equals(this.strType);
        result &= copy.routing.equals(this.routing);
        result &= copy.link.equals(this.link);
        return result;
    }
}
