package org.ow2.odis.model.cmd;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.decoder.IDecoder;
import org.ow2.odis.encoder.IEncoder;
import org.ow2.odis.util.CodingDecoding;
import org.ow2.odis.util.IntegerModif;

/**
 * The Class CmdEncoderDecoder.
 */
public class CmdEncoderDecoder implements IEncoder, IDecoder {
    // ----------------------------------------------------------------------------------------
    // attributs
    // ----------------------------------------------------------------------------------------
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(CmdEncoderDecoder.class.getName());

    // ----------------------------------------------------------------------------------------
    // methodes implementant IEncoder
    // ----------------------------------------------------------------------------------------
    /**
     * Encode.
     * @param object the object
     * @return the byte[]
     * @see com.francetelecom.odis.component.encoder.IEncoder#encode(java.lang.Object)
     */
    public byte[] encode(Object object) {
        Cmd cmd = (Cmd) object;
        int nbrParams = cmd.getMapParams().size();
        byte[][] value = new byte[9 + nbrParams * 2][];
        try {
            value[0] = CodingDecoding.longTo8Bytes(cmd.getTime());
            value[1] = CodingDecoding.stringToBytes(cmd.getDomaine());
            value[2] = CodingDecoding.intTo4Bytes(cmd.getIdLocalization());
            value[3] = CodingDecoding.stringToBytes(cmd.getIdSender());
            value[4] = CodingDecoding.stringToBytes(cmd.getIdReceiver());
            value[5] = CodingDecoding.intTo4Bytes(cmd.getIdCmd());
            value[6] = CodingDecoding.intTo4Bytes(cmd.getCmd());
            value[7] = CodingDecoding.bytesToBytes(cmd.getByteBuffer());
            value[8] = CodingDecoding.intTo4Bytes(nbrParams);
            Set setKey = cmd.getMapParams().keySet();
            Object[] keys = setKey.toArray();
            for (int i = 0; i < nbrParams; i++) {
                value[9 + i * 2] = CodingDecoding.stringToBytes(keys[i].toString());
                value[9 + i * 2 + 1] = CodingDecoding.stringToBytes(cmd.getMapParams().get(keys[i]));
            }
        } catch (UnsupportedEncodingException e) {
            LOGGER.log(BasicLevel.ERROR, "UnsupportedEncodingException", e);
        } catch (Exception e) {
            LOGGER.log(BasicLevel.ERROR, "Exception", e);
        }
        return CodingDecoding.getLinearByte(value);
    }

    // ----------------------------------------------------------------------------------------
    // methodes implementant IDecoder
    // ----------------------------------------------------------------------------------------
    /**
     * Decode.
     * @param msg the msg
     * @return the list
     * @see com.francetelecom.odis.component.decoder.IDecoder#decode(byte[])
     */
    public List decode(byte[] msg) {
        List result = new ArrayList();
        IntegerModif index = new IntegerModif(0);
        try {
            Cmd cmd;
            String param;
            String key;
            while (index.intValue() < msg.length) {
                cmd = new Cmd();
                cmd.setTime(CodingDecoding.getLongFrom8bytes(msg, index));
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, "time :" + cmd.getTime());
                }
                cmd.setDomaine(CodingDecoding.getString(msg, index));
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, "domaine :" + cmd.getDomaine());
                }
                cmd.setIdLocalization(CodingDecoding.getIntFrom4bytes(msg, index));
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, "id localisation :" + cmd.getIdLocalization());
                }
                cmd.setIdSender(CodingDecoding.getString(msg, index));
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, "id sender :" + cmd.getIdSender());
                }
                cmd.setIdReceiver(CodingDecoding.getString(msg, index));
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, "id receiver :" + cmd.getIdReceiver());
                }
                cmd.setIdCmd(CodingDecoding.getIntFrom4bytes(msg, index));
                cmd.setCmd(CodingDecoding.getIntFrom4bytes(msg, index));
                cmd.setByteBuffer(CodingDecoding.getBytesSecure(msg, index));
                int nbrParams = CodingDecoding.getIntFrom4bytes(msg, index);
                for (int i = 0; i < nbrParams; i++) {
                    key = CodingDecoding.getString(msg, index);
                    param = CodingDecoding.getString(msg, index);
                    cmd.getMapParams().put(key, param);
                }
                result.add(cmd);
            }
        } catch (UnsupportedEncodingException e) {
            LOGGER.log(BasicLevel.ERROR, "UnsupportedEncodingException", e);
        }
        return result;
    }

    // ----------------------------------------------------------------------------------------
    // methodes implementant IEncoder & IDecoder
    // ----------------------------------------------------------------------------------------
    /**
     * Gets the type.
     * @return the type
     * @see com.francetelecom.odis.component.encoder.IEncoder#getType()
     */
    public String getType() {
        return Cmd.class.getName();
    }
}
