package org.ow2.odis.node;
/**
 * JMX MBean interface of a Node.
 * @author Beyler Olivier FranceTelecom R&D
 */
public interface NodeMBean {
    /**
     * setter of the state of a node.
     * @param state int
     * @return boolean
     */
    boolean setState(int state);

    /**
     * getter.
     * @return String
     */
    String getDescription();

    /**
     * getter of number of procceed invocation
     * @return long
     */
    long getProceedInvocation();

    /**
     * number of pending object present into buffer in.
     * @return long
     */
    long getNumberOfPendingObject();

    /**
     * number of byte present into buffer in.
     * @return long
     */
    long getSizeOfPendingObject();

    /**
     * number of exception during decoding object.
     * @return long
     */
    long getDecodingError();

    /**
     * getter of the number of runtime exception during proceed.
     * @return long
     */
    long getProceedError();

    /**
     * getter of the number of start of sending error.
     * @return long
     */
    long getSendingError();

    /**
     * check if the engine out is able to send message.
     * @return boolean
     */
    boolean isAbleToSendMessage();

    /**
     * getter of the node's state.
     * @return String
     */
    String getState();

    /**
     * getter of the node's engine in state.
     * @return String
     */
    String getStateEngineIn();

    /**
     * getter of the node's engine proceed state.
     * @return String
     */
    String getStateEngineProceed();

    /**
     * getter of the node's engine out state.
     * @return String
     */
    String getStateEngineOut();

    /**
     * getter of the type of buffer out.
     * @return String[]
     */
    String[] getBufferOutType();

    /**
     * Gets the name.
     * @return the p name
     */
    String getPName();

    /**
     * Get the interval.
     * @return the interval
     */
    long getInterval();
}
