package org.ow2.odis.policy;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Base class of an policy.
 * @author Beyler Olivier FranceTelecom R&D
 */
public abstract class AbstractPolicy {
    /**
     * logger of this class.
     */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            AbstractPolicy.class.getName());

    /**
     * the list of element contained by a policy.
     */
    private final transient List lsPolicyElements;

    /**
     * Constructor.
     */
    public AbstractPolicy() {
        super();
        LOGGER.log(BasicLevel.DEBUG, "Create abstractpolicy");
        lsPolicyElements = new LinkedList();
    }

    /**
     * create a new policy by using an old.
     * @param oldPolicy the old policy
     */
    public void duplicate(final AbstractPolicy oldPolicy) {
        // transfert all the element from the old policy to the new created
        IPolicyElement element;
        while (!oldPolicy.lsPolicyElements.isEmpty()) {
            element = (IPolicyElement) oldPolicy.lsPolicyElements.remove(0);
            addPolicyElement(element);
        }
    }

    /**
     * add a element only once time.
     * @param element the element
     * @return true if succeed false if element is already present
     */
    public boolean addPolicyElement(final IPolicyElement element) {
        if (element==null){
            throw new RuntimeException("Try to add a null policy Element");

        }
        final boolean result = !lsPolicyElements.contains(element);
        if (result) {
            lsPolicyElements.add(element);
        }
        return result;
    }

    /**
     * remove a element.
     * @param element the element
     * @return true if succeed false if element wasn't present
     */
    public boolean removePolicyElement(final IPolicyElement element) {
        return lsPolicyElements.remove(element);
    }

    /**
     * Gets the policy element.
     * @return return elected link return null if no link can be elected.
     */
    public abstract IPolicyElement getPolicyElement();

    /**
     * As the id of an element can be change a map must not be used to perform this operation.
     * @param identity the identity
     * @return the element associated to this id
     */
    public IPolicyElement getLinkById(final String identity) {
        final Iterator itElement = lsPolicyElements.iterator();
        IPolicyElement elements;
        IPolicyElement result = null;
        while (itElement.hasNext() && result == null) {
            elements = (IPolicyElement) itElement.next();
            if (elements.getId().equals(identity)) {
                result = elements;
            }
        }
        return result;
    }

    /**
     * Returns the list of element contained by this policy.
     * @return list.
     */
    public List getLsPolicyElements() {
        return lsPolicyElements;
    }
}
