package org.ow2.odis.policy;

import java.util.Iterator;
import java.util.List;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * implement a round robin policy without regarding priority.
 * @author olivier beyler - orange-labs
 */
public class RoundRobinPolicy extends AbstractPolicy {
    /**
     * Constructor.
     */
    public RoundRobinPolicy() {
        super();
    }

    /**
     * Constructor by copy.
     * @param policy
     */
    public RoundRobinPolicy(AbstractPolicy policy) {
        this();
        duplicate(policy);
    }

    /**
     * @return the next elected IPolicyElement
     * @see com.francetelecom.odis.component.policy.AbstractPolicy#getPolicyElement()
     */
    public IPolicyElement getPolicyElement() {
        IPolicyElement result = null;
        final List lsElement = getLsPolicyElements();
        // si pas de lien on ressort
        if (lsElement.size() == 0) {
            if (LOGGER.isLoggable(BasicLevel.WARN)) {
                LOGGER
                        .log(BasicLevel.WARN,
                                "Request 'getPolicyElement' done on empty policy, no link has been registred");
            }
        } else {
            final Iterator itElement = lsElement.iterator();
            IPolicyElement element;
            int nbr;
            while (itElement.hasNext() && result == null) {
                element = (IPolicyElement) itElement.next();
                nbr = element.getPendingMessage();
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, element.toString() + " had "
                            + nbr + " waiting object");
                }
                if (nbr > 0) {
                    if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                        LOGGER.log(BasicLevel.DEBUG, "find "
                                + element.toString() + " with " + nbr
                                + " waiting object");
                    }
                    result = element;
                    // enleve l'element pour le replacer en fin de liste
                    lsElement.remove(element);
                    lsElement.add(element);
                }
            }
        }
        return result;
    }
}
