package org.ow2.odis.port;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.engine.AbstractEngine;
import org.ow2.odis.engine.EngineIn;
import org.ow2.odis.engine.EngineOut;
import org.ow2.odis.model.FromAttribute;
import org.ow2.odis.model.ToAttribute;

/**
 * Manage the creation of the link use inside a node.
 * @author olivier Beyler
 */
public class PortFactory {
    /**
     *
     */
    final static Logger LOGGER = Monolog.initialize().getLogger(PortFactory.class.getName());

    /**
     * map between link name and port In instance.
     */
    private Map mapPortInLinkName = new HashMap();

    /**
     * map between link name and port Out instance.
     */
    private Map mapPortOutLinkName = new HashMap();

    /**
     * reference on engine instance (In or Out).
     */
    AbstractEngine engine = null;

    /**
     * constructor.
     * @param engine
     */
    public PortFactory(AbstractEngine engine) {
        super();
        this.engine = engine;
    }

    /**
     * return associated portIn to name Create one if need
     * @param attribute FromAttribut
     * @return PortIn
     */
    public PortIn getPortIn(FromAttribute attribute) {
        PortIn portIn = (PortIn) mapPortInLinkName.get(attribute.getLink());
        if (portIn == null) {
            portIn = new PortIn(attribute);
            portIn.setEngine((EngineIn) engine);
            mapPortInLinkName.put(attribute.getLink(), portIn);
        }
        return portIn;
    }

    /**
     * return every portIn contain into PortFactory
     * @return collection
     */
    public Collection getPortIns() {
        return mapPortInLinkName.values();
    }

    /**
     * return associated portOut to name Create one if need using default portOut
     * @param attribute ToAttribute
     * @return PortOut
     */
    public PortOut getPortOut(ToAttribute attribute) {
        PortOut portOut = (PortOut) mapPortOutLinkName.get(attribute.getLink());
        if (portOut == null) {
            portOut = new PortOut(attribute, (EngineOut) engine);
            attribute.setPortOut(portOut);
            mapPortOutLinkName.put(attribute.getLink(), portOut);
        }
        return portOut;
    }

    /**
     * return every portOut contain into PortFactory.
     * @return collection
     */
    public Collection getPortOuts() {
        return mapPortOutLinkName.values();
    }
}
