package org.ow2.odis.proceed;

import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.engine.FWEvent;
import org.ow2.odis.engine.IProceed;

/**
 * The Class LogProceed.
 */
public class LogProceed implements IProceed, IDeclare {
    // --------------------------------------------------------------------------
    // --------------
    // attributs
    // --------------------------------------------------------------------------
    // --------------
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(
            LogProceed.class.getName());

    /** The consummable classes. */
    private final String[] consummableClasses = { Object.class.getName() };

    /** The generated classes. */
    private final String[] generatedClasses = { Object.class.getName() };

    // --------------------------------------------------------------------------
    // --------------
    // methodes
    // --------------------------------------------------------------------------
    // --------------
    /**
     * Instantiates a new log proceed.
     */
    public LogProceed() {
        super();
    }

    /**
     * implement IProceed interface forward the object pass as parameter.
     * @param obj the obj
     * @return the list
     */
    public List proceed(Object obj) {
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            if (obj != null) {
                LOGGER.log(BasicLevel.DEBUG, "proceed obj :" + obj.toString());
            } else {
                LOGGER.log(BasicLevel.DEBUG, "proceed obj without object");
            }
        }
        return null;
    }

    /**
     * Init.
     * @param value the value
     * @return boolean
     * @see com.francetelecom.odis.component.engine.IProceed#init(java.lang.String)
     */
    public boolean init(String value) {
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "init logprocess by " + value
                    + " properties file");
        }
        return true;
    }

    /**
     * Close.
     * @see com.francetelecom.odis.component.engine.IProceed#close()
     */
    public void close() {
    }

    /**
     * Notify FW event.
     * @param event the event
     * @see com.francetelecom.odis.component.engine.IProceed#notifyFWEvent(com.francetelecom.odis.component.engine.FWEvent)
     */
    public void notifyFWEvent(FWEvent event) {
    }

    // --------------------------------------------------------------------------
    // --------------
    // Getter and Setter for each private attribute
    // --------------------------------------------------------------------------
    // --------------
    /**
     * @return String[]
     * @see com.francetelecom.odis.component.proceed.IDeclare#getConsumableClasses()
     */
    public String[] getConsumableClasses() {
        return consummableClasses;
    }

    /**
     * @return String[]
     * @see com.francetelecom.odis.component.proceed.IDeclare#getGenerateClasses()
     */
    public String[] getGenerateClasses() {
        return generatedClasses;
    }
}
