package org.ow2.odis.routing;

import org.ow2.odis.connection.context.Context;

/**
 * An ODIS Destination denotes where data can be sent.<br>
 * A simple Destination only includes a String, which can represent an object type or an id for explicit routing by the proceed. A
 * contextual destination includes a Context, allowing routing towards one specific client of a contextual connection.
 * @author ojoh6423
 */
public class Destination {
    /** Object type or explicit routing id */
    private String destinationName = null;

    /** Context of the recipient */
    private Context destinationContext = null;

    /**
     * Constructor for a simple Destination
     * @param dest the object type or explicit routing id
     */
    public Destination(String dest) {
        destinationName = dest;
    }

    /**
     * Constructor for a contextual Destination
     * @param dest the object type or explicit routing id
     * @param ctx the context of the recipient
     */
    public Destination(String dest, Context ctx) {
        destinationName = dest;
        destinationContext = ctx;
    }

    /**
     * Checks the existence of a Context
     * @return
     */
    public boolean isContextual() {
        return (destinationContext != null);
    }

    /**
     * Getter for the simple destination
     * @return
     */
    public String getName() {
        return destinationName;
    }

    /**
     * Getter for the context of the destination
     * @return
     */
    public Context getContext() {
        return destinationContext;
    }

    /**
     * Setter for the simple destination
     * @param name
     */
    public void setName(String name) {
        destinationName = name;
    }

    /**
     * Returns a String representation of the Destination including destination name and context.
     * @return a String representation of the Destination
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(getClass().getName());
        sb.append('@').append(destinationName).append('/').append(
                destinationContext);
        return sb.toString();
    }
}
