package org.ow2.odis.routing;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.model.Const;

/**
 * The Class RoutingObject.
 */
public class RoutingObject {
    /** The payload. */
    private Object payload = null;

    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            RoutingObject.class.getName());

    /** The list of destinations. */
    private List lsDestinations = null;

    /** The factory. */
    private RoutingObjectFactory factory = null;

    /**
     * Instantiates a new routing object.
     * @param factory the factory
     */
    public RoutingObject(RoutingObjectFactory factory) {
        super();
        this.factory = factory;
        lsDestinations = new ArrayList();
    }

    /**
     * Gets the list destinations.
     * @return the list of destination witch be assign to this object
     */
    public Collection getLsDestinations() {
        return lsDestinations;
    }

    /**
     * Adds a simple destination by its name to the routing object.
     * @param filter the name of the wanted destination
     * @return true if the destination is found
     */
    public boolean addDestination(String filter) {
        String translatedDestination = factory.translateDestination(filter);
        if (translatedDestination != null) {
            lsDestinations.add(new Destination(translatedDestination));
            return true;
        }
        traceErrorNoDestinationFound(filter);
        return false;
    }

    private void traceErrorNoDestinationFound(String dest) {
        StringBuffer errBuff = new StringBuffer("Unable to find destination : ");
        errBuff.append(dest);
        errBuff.append(Const.EOL);
        Collection t = factory.mapTranslatedDestination.keySet();
        if (t.isEmpty()) {
            errBuff.append("cause no destination are avialable");
        } else {
            errBuff.append("only available destination(s) : ");
            errBuff.append(Const.EOL);
            Iterator it = t.iterator();
            while (it.hasNext()) {
                errBuff.append(it.next().toString());
                errBuff.append(Const.EOL);
            }
            LOGGER.log(BasicLevel.ERROR, errBuff.toString());
        }
    }

    /**
     * Adds a full destination to the routing object.
     * @param dest
     * @return true if the destination is found
     */
    public boolean addDestination(Destination dest) {
        String translatedDestination = factory.translateDestination(dest
                .getName());
        if (translatedDestination != null) {
            dest.setName(translatedDestination);
            lsDestinations.add(dest);
            return true;
        }
        traceErrorNoDestinationFound(dest.getName());
        return false;
    }

    /**
     * Clear destination.
     */
    public void clearDestination() {
        lsDestinations.clear();
    }

    /**
     * All destinations.
     */
    public void allDestinations() {
        // TODO : used by business ? what is the contract ?
        Collection col = factory.getLsDestinations();
        Iterator it = col.iterator();
        while (it.hasNext()) {
            lsDestinations.add(it.next());
        }
    }

    /**
     * All all destinations.
     * @deprecated use allDestinations instead of allAllDestination
     */
    public void allAllDestinations() {
        Collection col = factory.getLsDestinations();
        Iterator it = col.iterator();
        while (it.hasNext()) {
            lsDestinations.add(it.next());
        }
    }

    /**
     * Sets the payload. *
     * @param payload the new payload
     */
    public void setPayload(Object payload) {
        this.payload = payload;
    }

    /**
     * Gets the payload.
     * @return the payload
     */
    public Object getPayload() {
        return payload;
    }
}
