package org.ow2.odis.routing;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * implement Factory pattern to control the creation of routingObject.
 */
public class RoutingObjectFactory {
    /**
     * logger of the class.
     */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            RoutingObjectFactory.class.getName());

    /**
     * contain the translation of all destinations.
     */
    Map mapTranslatedDestination = null;

    /**
     * Constructor.
     */
    private RoutingObjectFactory() {
        super();
        mapTranslatedDestination = new HashMap();
    };

    /**
     * Create a routing factory and initialize the translate destination map.
     * @param routing object witch have a list of destination.
     * @param properties property witch allow to translate destination.
     */
    public RoutingObjectFactory(IRouting routing, Properties properties) {
        this();
        computeTranslatedDestinations(routing, properties);
    };

    /**
     * Constructor.
     * @param routing object witch have a list of destination.
     */
    public RoutingObjectFactory(IRouting routing) {
        this();
        computeTranslatedDestinations(routing, null);
    };

    /**
     * creator of routing object.
     * @return a new routing object.
     */
    public RoutingObject newRoutingObject() {
        return new RoutingObject(this);
    };

    /**
     * accessor.
     * @return a collection witch represent all destination after beeing translated.
     */
    protected Collection getLsDestinations() {
        return mapTranslatedDestination.values();
    }

    /**
     * Convert the destination by using a propery information. If the property is unknown there is no translation.
     * @param routing object witch have a list of destination.
     * @param properties : properties of translation.
     */
    private void computeTranslatedDestinations(IRouting routing,
            Properties properties) {
        if (routing != null) {
            Iterator itDestination = routing.getLsDestinations().iterator();
            String destination;
            String translate;
            while (itDestination.hasNext()) {
                destination = itDestination.next().toString();
                if (properties == null) {
                    mapTranslatedDestination.put(destination, destination);
                } else {
                    translate = properties.getProperty(destination);
                    if (translate == null) {
                        LOGGER.log(BasicLevel.ERROR,
                                "Unable to translate destination "
                                        + destination);
                    } else {
                        mapTranslatedDestination.put(destination, translate);
                    }
                }
            }
        } else {
            LOGGER
                    .log(BasicLevel.ERROR,
                            "Unable to create a RoutingObjectFactory no destination will be created");
        }
    }

    /**
     * convert a destination into another destination
     * @param destination String
     * @return String
     */
    public String translateDestination(String destination) {
        String result = (String) mapTranslatedDestination.get(destination);
        return result;
    }
}
