package org.ow2.odis.util;

import java.util.Iterator;
import java.util.List;

import org.jdom.Element;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.domain.Domain;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisFactoryException;

/**
 * The Class CheckUtilities.
 * @author Beyler Olivier FranceTelecom R&D utility class
 */
public class CheckUtilities {

    /**
     * create a className instance.
     * @param className the class name
     * @param version
     * @param resumeName the resume name
     * @param logger log
     * @return className instance
     * @throws OdisAttributeException if the class is not into the class path or if the constructor is hidden.
     */
    public static Object createInstance(final String className,
            final String version, final String resumeName, final Logger logger)
            throws OdisAttributeException {
        Object result = null;
        try {
            result = Domain.getInstance().getFactoryInstance().getInstanceOf(
                    className, version);
            if (logger.isLoggable(BasicLevel.DEBUG)) {
                StringBuffer buff = new StringBuffer(resumeName);
                buff.append("create class \"");
                buff.append(className);
                buff.append("\" in version \"");
                buff.append(version);
                buff.append("\"");
                logger.log(BasicLevel.DEBUG, buff.toString());
            }
        } catch (OdisFactoryException e) {
            throw new OdisAttributeException(e);
        }
        return result;
    }

    /**
     * Log a warn message if properties are unused.
     * @param lsUnusedProperties list of unused properties
     * @param logger log
     */
    public static void checkUnusedProperties(final List lsUnusedProperties,
            final Logger logger) {
        if (!lsUnusedProperties.isEmpty() && logger.isLoggable(BasicLevel.WARN)) {
            Element element = ((Element) lsUnusedProperties.get(0))
                    .getParentElement();
            String jNode = element.getName();
            StringBuffer buffer = new StringBuffer("");
            Element lastElement = element.getParentElement();
            String name;
            while (lastElement != null && !lastElement.isRootElement()) {
                buffer.insert(0, "->");
                buffer.insert(0, lastElement.getName());
                name = lastElement.getAttributeValue("name");
                if (name != null) {
                    buffer.insert(0, "] ");
                    buffer.insert(0, name);
                    buffer.insert(0, "[");
                }
                lastElement = lastElement.getParentElement();
            }
            Iterator itChildren = lsUnusedProperties.iterator();
            while (itChildren.hasNext()) {
                element = (Element) itChildren.next();
                logger.log(BasicLevel.WARN, buffer.toString() + jNode + "->"
                        + "\"" + element.getName()
                        + "\" unable to set this property.");
            }
        }
    }
}
