package org.ow2.odis.util;
/**
 * Implement a Semaphore to help thread synchronization.
 * @author Beyler Olivier FranceTelecom R&D
 */
public class Semaphore {
    /**
     * internal count
     */
    private int value;
    
    int getValue(){
        return value;
    }
    /**
     * Constructor.
     */
    public Semaphore() {
        super();
        value = 0;
    }

    /**
     * free the semaphore to restart all thread stopped on thread.
     */
    public void release() {
        synchronized (this) {
            ++value;
            // debloque les threads qui attendent cet evenement:
            notifyAll();
        }
    }

    /**
     * stop the current thread until an other release the semaphore.
     * @throws InterruptedException the interrupted exception
     */
    public void waiting() throws InterruptedException {
        waiting(0);
    }

    /**
     * stop the current thread until an other release the semaphore.
     * @param timeOut
     * @throws InterruptedException the interrupted exception
     */
    public void waiting(long timeOut) throws InterruptedException {
        synchronized (this) {
            while (value <= 0) {
                wait(timeOut); // bloque jusqu'a un notify()
            }
            value--;
        }
    }
}
